/*
 * Copyright (C) 2003  Diez B. Roggisch [deets@web.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: AbstractPage.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.dbroggisch.page;

import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import javax.servlet.ServletException;

import org.apache.log4j.Logger;
import org.barracudamvc.core.comp.BComponent;
import org.barracudamvc.core.comp.ViewContext;
import org.barracudamvc.core.event.BaseEvent;
import org.barracudamvc.core.event.ControlEventContext;
import org.barracudamvc.core.event.DispatchQueue;
import org.barracudamvc.core.event.EventException;
import org.barracudamvc.core.event.ViewEventContext;
import org.barracudamvc.core.event.helper.DefaultViewHandler;
import org.w3c.dom.Document;


/**
 * @author <a href="mailto:deets@web.de">Diez B. Roggisch</a>
 * @version 1.0
 */
public abstract class AbstractPage extends DefaultViewHandler implements Page {

    private static final Logger logger = Logger.getLogger(AbstractPage.class.getName());
    private List dEvents;

    /**
     * Describe <code>setDefaultEvents</code> method here.
     *
     * @param events a <code>List</code> value
     */
    public void setDefaultEvents(List events) {
        dEvents = events;
    }

    /**
     * Describe <code>createDefaultModels</code> method here.
     *
     * @param ctx a <code>ControlEventContext</code> value
     */
    public void createDefaultModels(ControlEventContext ctx) {
        filterEvents(ctx, dEvents);
    }

    /**
     * Return a Document instance. This is called once before the rendering takes place and
     * should return a new instance.
     *
     * @param iLocale the <code>Locale</code> of the client for locale specific documents
     * @return a <code>Document</code> value
     */
    public abstract Document getDocument(Locale iLocale);

    // handleviewevent is functionality that we don't necessarily want to throw
    // away.. but the whole render/handleviewevent thing is confusing. so we hide it here.
    // overriding the render method gives the old behavior.
    /**
     * Describe <code>handleViewEvent</code> method here.
     *
     * @param root a <code>BComponent</code> value
     * @return a <code>Document</code> value
     * @exception EventException if an error occurs
     * @exception ServletException if an error occurs
     * @exception IOException if an error occurs
     */
    public Document handleViewEvent(BComponent root)
            throws EventException, ServletException, IOException {
        ViewContext ctx = getViewContext();
        Locale iLocale = ctx.getViewCapabilities().getClientLocale();
        if (logger.isDebugEnabled()) logger.debug("Locale sent by the client is: " + iLocale);
        Document doc = getDocument(iLocale);
        if (logger.isInfoEnabled()) logger.info("Created document " + doc.getClass().getName());
        render(root, ctx, doc);
        return doc;
    }

    /**
     * Describe <code>render</code> method here.
     *
     * @param vec a <code>ViewEventContext</code> value
     * @exception EventException if an error occurs
     * @exception ServletException if an error occurs
     * @exception IOException if an error occurs
     */
    public void render(ViewEventContext vec)
            throws EventException, ServletException, IOException {
        handleViewEvent(vec);
    }

    /**
     * Overload this method to perform the actual process of
     * rendering.
     *
     * @param root a <code>BComponent</code> value you have to add your BComponents to.
     * @param vec a <code>ViewContext</code> value
     * @param doc a <code>Document</code> value which is obtained by calling getDocument(Locale)
     * @exception EventException if an error occurs
     * @exception ServletException if an error occurs
     * @exception IOException if an error occurs
     */
    public abstract void render(BComponent root, ViewContext vec, Document doc)
            throws EventException, ServletException, IOException;

    /**
     * Describe <code>filterEvents</code> method here.
     *
     * @param ctx a <code>ControlEventContext</code> value
     * @param defaultEvents a <code>List</code> value
     */
    public static void filterEvents(ControlEventContext ctx, List defaultEvents) {
        if (defaultEvents != null) {
            BaseEvent event = ctx.getEvent();

            for (Iterator it = defaultEvents.iterator(); it.hasNext();) {
                if (((BaseEvent)it.next()).equals(event)) {
                    if (logger.isDebugEnabled()) logger.debug("Removing event " + event.getClass().getName() + " from the queue");
                    it.remove();
                }
            }

            DispatchQueue queue = ctx.getQueue();

            for (Iterator it = defaultEvents.iterator(); it.hasNext();) {
                //add remaining events (less the current event) back to the queue
                queue.addEvent((BaseEvent)it.next());
            }
        }
    }

}
