/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: DefaultViewContext.java 120 2005-12-22 19:09:56Z christianc $
 */
package org.barracudamvc.core.comp;

import java.util.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.barracudamvc.core.event.EventContext;
import org.barracudamvc.core.event.ViewEventContext;
import org.barracudamvc.core.view.ViewCapabilities;
import org.barracudamvc.plankton.data.DefaultStateMap;
import org.barracudamvc.plankton.data.StateMap;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * This class provides the default implementation of a ViewContext. A View
 * Context basically exists to provide the component models with the 
 * information they need to actually pass back the proper data to the 
 * component.
 */
public class DefaultViewContext implements ViewContext {

    //private vars
    private StateMap statemap = new DefaultStateMap();

    //--------------- Constructors -------------------------------
    /**
     * Create a DefaultViewContext
     */
    public DefaultViewContext() {}
    
    /**
     * Create a DefaultViewContext for req/response object
     */
    public DefaultViewContext(HttpServletRequest ireq, HttpServletResponse iresp) {
        this(null, ireq, iresp);
    }
    
    /**
     * Create a DefaultViewContext for a specific ViewCapabilities obj
     */
    public DefaultViewContext(ViewCapabilities ivc, HttpServletRequest ireq, HttpServletResponse iresp) {
        if (ivc==null) ivc = new ViewCapabilities(ireq, iresp);
        this.putState(ViewContext.VIEW_CAPABILITIES, ivc);
        this.putState(ViewContext.REQUEST, ireq);
        this.putState(ViewContext.RESPONSE, iresp);
    }
    
    /**
     * Create a DefaultViewContext for a specific EventContext
     */
    public DefaultViewContext(ViewEventContext ivec) {
        this.putState(ViewContext.VIEW_CAPABILITIES, ivec.getViewCapabilities());
        this.putState(ViewContext.EVENT_CONTEXT, ivec);
        this.putState(ViewContext.REQUEST, ivec.getRequest());
        this.putState(ViewContext.RESPONSE, ivec.getResponse());
    }



    //--------------- DefaultViewContext -------------------------
    /**
     * Get the underlying ViewCapabilities object
     *
     * @return the underlying ViewCapabilities object
     */
    public ViewCapabilities getViewCapabilities() {
        return (ViewCapabilities) this.getState(ViewContext.VIEW_CAPABILITIES);
    }

    /**
     * Get the underlying EventContext object
     *
     * @return the underlying EventContext object
     */
    public EventContext getEventContext() {
        return (EventContext) this.getState(ViewContext.EVENT_CONTEXT);
    }

    /**
     * Get the underlying ElementFactory object (if it exists)
     *
     * @return the underlying ElementFactory object (if it exists)
     * @deprecated
     */
//csc_012605_1 - removed
/*
    public ElementFactory getElementFactory() {
        return (ElementFactory) this.getState(ViewContext.ELEMENT_FACTORY);
    }
*/
    
    /**
     * Get the underlying template Node (if it exists)
     *
     * @return the underlying template Node (if it exists)
     */
    public Node getTemplateNode() {
        return (Node) this.getState(ViewContext.TEMPLATE_NODE);
    }

    //csc_072604_2 - added
    /**
     * Set the underlying template Document (note that this method is NOT part of 
     * the ViewContext interface)
     */
    public void setDocument(Document document) {
        this.putState(ViewContext.DOCUMENT, document);
    }

    //csc_072604_2 - added
    /**
     * Get the underlying template Document (if it exists - note that
     * the ViewContext may be constructed BEFORE the Document has actually
     * been loaded yet, so this property may not be available immediately
     * after object creation. It should always be set, however, by the time
     * models are actually using the ViewContext).
     *
     * @return the underlying template Document (if it exists)
     */
    public Document getDocument() {
        return (Document) this.getState(ViewContext.DOCUMENT);
    }

    /**
     * Get the underlying HttpServletRequest
     *
     * @return the underlying HttpServletRequest
     */
    public HttpServletRequest getRequest() {
        return (HttpServletRequest) this.getState(ViewContext.REQUEST);
    }

    /**
     * Get the underlying HttpServletResponse
     *
     * @return the underlying HttpServletResponse
     */
    public HttpServletResponse getResponse() {
        return (HttpServletResponse) this.getState(ViewContext.RESPONSE);
    }


    
    //-------------------- StateMap ------------------------------
    /**
     * set a property in this StateMap
     *
     * @param key the state key object
     * @param val the state value object
     */    
    public void putState(Object key, Object val) {
        statemap.putState(key,val);
    }
    
    /**
     * get a property in this StateMap
     *
     * @param key the state key object
     * @return the value for the given key
     */    
    public Object getState(Object key) {
        return statemap.getState(key);
    }
    
    /**
     * remove a property in this StateMap
     *
     * @param key the key object
     * @return the object which was removed
     */    
    public Object removeState(Object key) {
        return statemap.removeState(key);
    }
    
    /**
     * get a keyset for this StateMap (whether or 
     * not the set is backed by the data store depends on 
     * the implementation)
     *
     * @return a Set of keys for this StateMap
     */
    public Set getStateKeys() {
        return statemap.getStateKeys();
    }
    
    /**
     * get a copy of the underlying Map
     *
     * @return a copy of the underlying state Map
     */    
    public Map getStateStore() {
        return statemap.getStateStore();    
    }
    
    //csc_052803_2 - added
    /**
     * clear all state information
     */
    public void clearState() {
        statemap.clearState();
    }
    
}