/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: ApplicationGateway.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.core.event;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.*;
import org.barracudamvc.core.event.events.ActionEvent;
import org.barracudamvc.core.event.events.LongRunningEventGateway;
import org.barracudamvc.core.helper.servlet.BarracudaServletRequestWrapper;
import org.barracudamvc.core.helper.servlet.BarracudaServletResponseWrapper;
import org.barracudamvc.core.helper.servlet.DefaultServletRequestWrapper;
import org.barracudamvc.core.helper.servlet.DefaultServletResponseWrapper;
import org.barracudamvc.core.helper.servlet.DeferredResponseWrapper;
import org.barracudamvc.core.helper.servlet.RequestWrapper;
import org.barracudamvc.core.helper.servlet.ResponseWrapper;
import org.barracudamvc.core.helper.servlet.ScriptDetector;
import org.barracudamvc.core.view.ViewCapabilities;
import org.barracudamvc.plankton.*;
import org.barracudamvc.plankton.data.*;
import org.barracudamvc.plankton.exceptions.ExceptionUtil;
import org.barracudamvc.plankton.exceptions.NestableException;
import org.barracudamvc.plankton.http.ServletUtil;
import org.barracudamvc.plankton.http.URLRewriter;

/**
 * <p>The application gateway is responsible for a number of things.
 *
 * <ol>
 *   <li>It acts as a gateway servlet for all event handlers in this
 *         application.</li>
 *
 *   <li>It defines and instantiates a number of entities needed to
 *       dispatch events (EventBroker, EventPool, DispatcherFactory, etc)</li>
 *
 *      <li>It defines which event extension we are using</li>
 *
 *      <li>It registers all EventGateways, and any local Event interests</li>
 *
 *      <li>It performs the initial mapping of HTTPRequests to Events for this
 *       particular domain </li>
 * </ol>
 *
 * <p>Consequently, if you want to use the Barracuda event model, this
 * is the class that really kicks it all off. You must either extend this
 * class OR use the DefaultApplicationAssembler in order to specify that
 * this servlet handles all event requests for an application. This will
 * allow the system to convert requests to events and dispatch them through
 * the EventBroker to any listeners within EventGateways that have registered
 * interest with the broker.
 *
 * <p>This class should be the first servlet loaded in your web.xml file.
 *
 * <p>For an example of how to do this, look at
 * <strong>org.barracudamvc.examples.ex1.SampleApplicationGateway</strong>
 *
 * <p>You might also want to look at the <a href="sm_barracuda_event.gif">UML Class
 * diagram of the Event model classes.</a>
 *
 * <Note on MDC support. If you turn on logging for org.barracudamvc.core.event.ApplicationGateway,
 * the following MDC values will be available to your logging - INFO: event DEBUG: params
 *
 * @author  Christian Cryder <christianc@granitepeaks.com>
 * @author  Diez Roggisch <diez.roggisch@artnology.com>
 * @author  Jacob Kjome <hoju@visi.com>
 * @version %I%, %G%
 * @since   1.0
 */
public class ApplicationGateway extends HttpServlet implements EventGateway {

    private static final Class CLASS = ApplicationGateway.class;
    private static final Logger logger = Logger.getLogger(CLASS);
    private static final Logger orLogger = Logger.getLogger(CLASS.getName()+"_ORStackTrace");
    private static final Logger mdcLogger = Logger.getLogger("mdc."+CLASS.getName());

    //public vars
    //...configuration constants (set through ObjectRepositoryAssembler)
    public static boolean USE_EVENT_POOLING = true;             //use event pooling?
    public static boolean RESPOND_WITH_404 = false;             //respond with a 404 if the event being dispatched is invalid    //csc_100603_1
    public static RequestWrapper REQUEST_WRAPPER = null;        //dbr_021602
    public static ResponseWrapper RESPONSE_WRAPPER = null;      //csc_010404_1
    public static boolean ENABLE_LR_EVENTS = true;              //allow for long running events?
    public static String LR_OVERRIDE_KEY = "$lr_override";      //if this key = true in the request, the LongRunning interface will be ignored (makes it easy to override LongRunning with certain URL configurations)

    //...configuration constants (set through servlet init params)
    private static final String APPLICATION_ASSEMBLER = "ApplicationAssembler";
    private static final String ASSEMBLY_DESCRIPTOR = "AssemblyDescriptor";
    private static final String SAX_PARSER = "SAXParser";

    //...LocalObjectRepository constants (available for apps to access)
    public static final String HTTP_SERVLET_REQUEST = CLASS+".HttpServletRequest";                  //(HttpServletReques)
    public static final String HTTP_SERVLET_RESPONSE = CLASS+".HttpServletResponse";                //(HttpServletResponse)
    public static final String THREAD_POOL = CLASS+".ThreadPool";                                   //(ThreadPool)

    //...EventContext constants (available for apps to access)
    public static final String TARGET_EVENT_NAME = CLASS+".TargetEventName";                        //(String)
    public static final String EXTERNAL_CONTEXT_OBJ_NAME = CLASS+".ExternalContextObjName";         //(Object)

    //req params
    public static final String LR_ID = "$lrid";                                                     //(String)

    //custom lrmap name
    public static final String LR_OR = "$lror";                                                     //(String)
    public static final String LR_SOR = "$lrsor";                                                   //(String)


    //private vars
    private EventBroker masterEventBroker = null;
    private EventPool masterEventPool = null;
//    private static final String LONG_RUNNING_RESPONSE_INDICATOR = "$is_lr_resp";                    //csc_010404_1
    protected EventGateway eventGateway = (EventGateway) Classes.newInstance(A_Classes.DEFAULT_EVENT_GATEWAY);   //csc_060903_3
    protected List gateways = null;
    protected boolean virgin = true;    //true after init before any requests handled - added mostly because log4j stuff in init() method is all going to log, rather than console - csc



    /**
     * Public noargs constructor
     */
    public ApplicationGateway() {
        logger.info("Instantiating "+this);
    }


    //--------------- User Configurable Methods ------------------
    /**
     * <p>Perform any local initialization (this is where you should
     * add any other known EventGateways)
     */
    public void initializeLocal() {
    }

    /**
     * <p>Perform any local initialization (this is where you should
     * add any other known EventGateways)
     *
     * @param iconfig the ServletConfig object used to configure this servlet
     * @deprecated If your code is still attempting to extend this old method
     *      signature, you need to change it to use initializeLocal() instead.
     *      If you still need to get a reference to the servlet config, just
     *      call 'this.getServletConfig()'. This method is now final so that
     *      code which might still be using it will no longer compile.
     */
    public final void initializeLocal(ServletConfig iconfig) throws ServletException {
    }

    /**
     * <p>Perform any local cleanup (this is where you should
     * remove any known EventGateways)
     */
    public void destroyLocal() {
    }

    /**
     * <p>Provide an instance of the specific EventBroker we want to use.
     * Override this method if you'd like to use something other than
     * the DefaultEventBroker.
     *
     * @return a new instance of the EventBroker
     */
    public EventBroker getNewEventBrokerInstance() {
        if (logger.isDebugEnabled()) logger.debug("instantiating DefaultEventBroker");
        return new DefaultEventBroker(getDispatcherFactory(), getEventExtension());
    }

    /**
     * <p>Provide an instance of the specific EventPool we want to use.
     * Override this method if you'd like to use something other than
     * the DefaultEventPool.
     *
     * @return a new instance of the EventPool.
     */
//csc_060903_1_start
/*
    public EventPool getNewEventPoolInstance() {
        if (logger.isDebugEnabled()) logger.debug("instantiating DefaultEventPool");
        return new DefaultEventPool();
    }
*/
//csc_060903_1_end

    /**
     * <p>Provide an instance of the specific DispatchQueue we want to use.
     * Override this method if you'd like to use something other than
     * the DefaultDispatchQueue.
     *
     * @return a new instance of the DispatchQueue.
     */
    public DispatchQueue getNewDispatchQueueInstance() {
        if (logger.isDebugEnabled()) logger.debug("instantiating DefaultDispatchQueue");
        return new DefaultDispatchQueue(true);
    }

    /**
     * <p>Provide an instance of the specific EventDispatcher we want to use.
     * Override this method if you'd like to use something other than
     * DefaultEventBroker.
     *
     * @return a new instance of the DispatcherFactory
     */
    public DispatcherFactory getDispatcherFactory() {
        if (logger.isDebugEnabled()) logger.debug("instantiating DefaultDispatcherFactory");
        return new DefaultDispatcherFactory();
    }

    /**
     * <p>Indicate which event extension we are handling. By default the gateway
     * handles extensions of <strong>.event</strong>. If you wish to handle
     * a different function you should override this method to return the
     * value defined in the web.xml file.
     *
     * @return a string defining the event extension handled by this servlet
     */
    public String getEventExtension() {
        return ".event";
    }

    /**
     * <p>Handle the default HttpRequest. It will probably be rare for
     * developers to override this.
     *
     * <p>Basically, this method receives a request and attempts to map it
     * to a valid HttpRequestEvent. If the event is invalid, or if the
     * event is NOT an instance of HttpRequestEvent, then we simply create
     * a new instance of HttpRequestEvent and dispatch that instead. This
     * is a very important feature, because it allows us to define events
     * which are not accessible to the outside world -- the gateway will
     * only dispatch HttpRequest events, so you define your publically
     * accessible API by creating an event hierarchy that extends from
     * the HttpRequestEvent object.
     *
     * <p>Once we have a valid event, we dispatch it. In this case (because this
     * is an Http gateway) we must have a response, so if we catch an
     * UnhandledEventException, we will generate a default error message and
     * return.
     *
     * @param req the servlet request
     * @param resp the servlet response
     * @throws ServletException
     * @throws IOException
     */
    public void handleDefault(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefaultExt(req, resp, null);
    }

    /**
     * <p>Handle the default HttpRequest with the ability to add the provided
     * object into the context. It will probably be rare for
     * developers to override this.
     *
     * <p>Basically, this method receives a request and attempts to map it
     * to a valid HttpRequestEvent. If the event is invalid, or if the
     * event is NOT an instance of HttpRequestEvent, then we simply create
     * a new instance of HttpRequestEvent and dispatch that instead. This
     * is a very important feature, because it allows us to define events
     * which are not accessible to the outside world -- the gateway will
     * only dispatch HttpRequest events, so you define your publically
     * accessible API by creating an event hierarchy that extends from
     * the HttpRequestEvent object.
     *
     * <p>Once we have a valid event, we dispatch it. In this case (because this
     * is an Http gateway) we must have a response, so if we catch an
     * UnhandledEventException, we will generate a default error message and
     * return.
     *
     * @param req the servlet request
     * @param resp the servlet response
     * @param externalContextObj object to put into the context
     * @throws ServletException
     * @throws IOException
     */
    public void handleDefaultExt(HttpServletRequest req, HttpServletResponse resp, Object externalContextObj) throws ServletException, IOException {
        String ourl = req.getRequestURL().toString();
        String oqstr = req.getQueryString();
        long bmillis = 0;
        long smillis = 0;
        long emillis = 0;
        if (virgin) {
            if (logger.isInfoEnabled()) {
                logger.info("\n\n\n\n\n\n"+
                            "=======================================\n"+
                            "ApplicationGateway servlet was reloaded\n"+
                            "=======================================");
            }
            virgin = false;
        }
        if (logger.isInfoEnabled()) bmillis = System.currentTimeMillis();
        if (logger.isInfoEnabled()) {
            logger.info("\n\n\n"+
                        "--------------------------------------------------------------------------\n"+
                        "Handling incoming HTTP request: "+ourl+(oqstr!=null ? "?"+oqstr : "")+"\n"+
                        "--------------------------------------------------------------------------");
        }
        BaseEvent event = null;


        //start by getting the event broker and pool. We get local copies so
        //that if these constant vaules change while we're mid-method, it's not a problem
        EventBroker eventBroker = getEventBroker();
        EventPool eventPool = getEventPool();
        ViewCapabilities vc = new ViewCapabilities(req, resp);
        DefaultEventContext context = null;
        HttpServletResponse oresp = null;
        DeferredResponseWrapper dresp = null;
        String lrid = req.getParameter(LR_ID);
        boolean lrOverride = (!ENABLE_LR_EVENTS || "true".equalsIgnoreCase(req.getParameter(LR_OVERRIDE_KEY)));
        boolean useEventPool = USE_EVENT_POOLING;
        boolean eventCameFromPool = false;
        boolean allowEventAliasing = true;    //just do it always
        String target = null;

        try {
            //set up the Session ObjectRepository
            ObjectRepository.setupSessionRepository(req);                                   //csc_022101.1




            //csc_010404_1_start
            //before we do anything else, see if we are dealing with a long running response
            if (lrid!=null && !lrOverride) {
                //now since we have a lrid, there should be an object in the long running OR...
                ObjectRepository lror = ObjectRepository.getObjectRepository(LR_OR, ObjectRepository.DEFAULT);
                ObjectRepository lrsor = ObjectRepository.getObjectRepository(LR_SOR, ObjectRepository.SOFT_THREADSAFE);
                dresp = (DeferredResponseWrapper) lror.getState(lrid);
//System.out.println("dresp:"+dresp+" isReady():"+(dresp!=null ? dresp.isReady() : false));
//System.out.println("checking for lr objects in lror:");
//lror.printStackTrace();

//csc_042204_2_start - this is problematic, because it prevents a user from hitting CTRL-R and reloading the
//                     report. Consequently, if the lrid is already finished, just resubmit as a new request
/*
                //...first and foremost, make sure the long running stuff hasn't already finished
                if (!LongRunningEventGateway.isUIDActive(lrid)) {
                    LongRunningEventGateway.showInactiveUIDResults(lrid, resp);
                    return;

                //...next make sure there's a dresp obj; if there's not, just clear the lrid value and proceed
                //...(this effectively resubmits the request)
                } else if (dresp==null) {
                    if (logger.isInfoEnabled()) logger.info("DeferredResponseWrapper missing from lror - continuing as if $lrid=null (pass ??)");
*/
                //...if the lrid is already complete, the uer probably just hit Ctrl-R on a page that has already
                //been rendered via long running. If the dresp is null, something unexpected happened, In either 
                //case, we can't really proceed, so clear the $lrid value and proceed as if they are just resubmitting 
                //the request
                if (!LongRunningEventGateway.isUIDActive(lrid) || dresp==null) {
                    if (logger.isInfoEnabled()) {
                        if (dresp==null) logger.info("DeferredResponseWrapper missing from lror - continuing as if $lrid=null (pass ??)");
                        else logger.info("lrid already complete (the user probably hit CTRL-R - continuing as if $lrid=null (pass ??)");
                    }
//csc_042204_2_end
                    ((BarracudaServletRequestWrapper) req).removeParameter(LR_ID);
                    String lridstr = LR_ID+"="+lrid;
//csc_022205_1_start - the order of these needs to be reversed, or the second line will never work (this was causing a double question mark ("??") in the url string
/*
                    oqstr = StringUtil.replace(oqstr, lridstr, "");
                    oqstr = StringUtil.replace(oqstr, "?"+lridstr, "");
*/
                    oqstr = StringUtil.replace(oqstr, "?"+lridstr, "");
                    oqstr = StringUtil.replace(oqstr, lridstr, "");
//csc_022205_1_end
                    lrid = null;

                    //now, check to see if there are any params for this url in the long running soft OR; if so,
                    //we can assume they did a CTRL-R on the page (that's why we're in this particular block) -
                    //in this case, restore the params from the last POST request
                    BarracudaServletRequestWrapper breq = (BarracudaServletRequestWrapper) req;
                    Map pmap = (Map) lrsor.getState(ourl);
                    if (pmap!=null) {
                        if (logger.isInfoEnabled()) logger.info("Restoring params from previous POST");
                        Iterator it = pmap.entrySet().iterator();
                        while (it.hasNext()) {
                            Map.Entry me = (Map.Entry) it.next();
                            String key = (String) me.getKey();
                            if (key.equals(LR_ID)) continue;
                            
//csc_022205_1_start
                            //if the query string contains the key in question, do not restore the value from the previous post.
                            //this was causing a problem which the check below was not catching - namely, a key which is only
                            //supposed to have one value (ie. not be an array) had key/val1 in the URL, and key/val2 in the previous 
                            //post cache. In other words, because the values were DIFFERENT, the code below would go ahead and 
                            //put them both in, meaning they got stuck in an array rather than as their base type, and then when 
                            //your application code (which is expecting a single instance of base type) tries to get them out, it gets a 
                            //class casting exception. This basically says, if the value is in the url string, don't restore it from 
                            //previous post. Should fix the problem without breaking anything else...
                            //
                            //(Note that at the root of this is the question - why are we getting a lrid request when the dresp is null???
                            //I don't know the answer to this question. May have to do with the user hitting the refresh at just the wrong 
                            //moment. We were encountering this problem in our app 0-5 times per day, so it was a very infrequent issue)
                            if (oqstr.indexOf("&"+key+"=")>0 || oqstr.indexOf("?"+key+"=")>0) continue;
//csc_022205_1_end
                            
                            String[] vals = (String[]) me.getValue();
                            for (int i=0; i<vals.length; i++) {
                                String val = vals[i];
                                String oval = breq.getParameter(key);
                                if (val==oval || (val!=null && val.equals(oval))) continue; //in certain situations, the url may actually have parameters embedded into it; those should not be reconstituted from the lror since they are already there
                                breq.addParameter(key, val);
                            }
                        }
                        breq.setMethod("POST");
                    }

                //...if there is, but the response has already been generated, just show it
//                } else if (dresp.isCommitted()) {
                } else if (dresp.isReady()) {
                    try {
                        //at this point, we want to flag the lrid as inactive (so that any subsequent requests
                        //to get status or cancel will be ignored, since we're displaying the results now)
                        LongRunningEventGateway.markUIDAsComplete(lrid);
                    
                        //first see if there is any error; if so - resend it
                        Integer error = (Integer) lror.getState(lrid+"_error");
                        if (error!=null) {
                            resp.sendError(error.intValue());
                            return;
                        }

                        //next check and see if we encountered an exception; if so, re-throw it
                        Exception dexcp = (Exception) lror.getState(lrid+"_exception");
                        if (dexcp!=null) {
                            if (logger.isInfoEnabled()) logger.info("Throwing a deferred EventException (pass 3)");
                            if (dexcp instanceof EventException) throw (EventException) dexcp;
                            else if (dexcp instanceof ServletException) throw (ServletException) dexcp;
                            else if (dexcp instanceof IOException) throw (IOException) dexcp;
                            else if (dexcp instanceof RuntimeException) throw (RuntimeException) dexcp;
                            else {
                                logger.warn("found unexpected exception in lror:"+dexcp);
                            }
                        }

                        //otherwise, render
                        if (logger.isInfoEnabled()) logger.info("Actually rendering completed response from DeferredResponseWrapper (pass 3)");
                        dresp.actuallySendResponse(resp);
                        return;
                    } finally {
                        lror.removeState(lrid+"*");
                        dresp = null;
                    }

                //...otherwise, lets set everything up to execute the long running task
                } else {
                    if (logger.isInfoEnabled()) logger.info("Writing response to DeferredResponseWrapper in lror (pass 2)");

                    //save a reference to our orig response; we'll use the deferred response to actually save the results
                    oresp = resp;
                    resp = dresp;
                    dresp.setUnderlyingResponse(oresp, lrid);

                    //we also want to reconstitute params from the lror (this ensures that
                    //all the params from the orig request are available this second time around)
                    BarracudaServletRequestWrapper breq = (BarracudaServletRequestWrapper) req;
                    Map pmap = (Map) lror.getState(lrid+"_params");
                    Iterator it = pmap.entrySet().iterator();
                    while (it.hasNext()) {
                        Map.Entry me = (Map.Entry) it.next();
                        String key = (String) me.getKey();
                        String[] vals = (String[]) me.getValue();
                        for (int i=0; i<vals.length; i++) {
                            String val = vals[i];
                            String oval = breq.getParameter(key);
                            if (val==oval || (val!=null && val.equals(oval))) continue; //in certain situations, the url may actually have parameters embedded into it; those should not be reconstituted from the lror since they are already there
                            breq.addParameter(key, val);
                        }
                    }
                    lror.removeState(lrid+"_params");

                    //now, if the original method was post, save the params in the soft lror (so in case they hit CTRL-R
                    //later, we at least have a chance of resubmitting properly)
                    if ("POST".equalsIgnoreCase((String) lror.getState(lrid+"_orig_method"))) {
                        lrsor.putState(ourl, req.getParameterMap());
                    } else {
                        lrsor.removeState(ourl);
                    }

/*    
                    //csc_031504_1_start
                    //now before we go any further, tell the browser that this particular
                    //response is done. This is very important, since IE and MZ can only 
                    //create 2 simultaneous connections. By writing this response here,
                    //the browser thinks we're done (even though this thread will continue
                    //on on the server until cancelled or complete). This then allows the 
                    //subsequent connections to occur to check the status and to wait for
                    //a response.                
                    oresp.setStatus(HttpServletResponse.SC_OK);  
                    oresp.setContentType("text/html");
                    StringBuffer sb = new StringBuffer(500);
                    sb.append("<html><body>Ok</body></html>");
                    String content = sb.toString();
                    oresp.setContentLength(content.length());        
                    Writer wr = oresp.getWriter();
                    wr.write(content);
                    wr.close();
                    oresp.flushBuffer();
                    //csc_031504_1_end
*/                    
                }
            }
            //csc_010404_1_end






            //store the req/resp objects in the local repository
            ObjectRepository lor = ObjectRepository.getLocalRepository();                   //csc_010404_1
            lor.putState(HTTP_SERVLET_REQUEST, req);                                        //csc_010404_1
            lor.putState(HTTP_SERVLET_RESPONSE, resp);                                      //csc_010404_1

            //figure out the name of the event handler being requested
            target = req.getServletPath();
            if (logger.isDebugEnabled()) {
                logger.debug("Incoming URI:"+req.getRequestURI());
                logger.debug("ServletPath:"+target);
//                logger.debug("HTTP Headers...");        //csc_052803.1
                ServletUtil.showHeader(req, logger);    //csc_052803.1
            }
            if (logger.isInfoEnabled()) ServletUtil.showParams(req, logger);                //csc_010404_1
//csc_062405_1_start
            if (mdcLogger.isDebugEnabled()) {
                StringBuffer sb = new StringBuffer(200);
                String sep = "?";
                Map pmap = req.getParameterMap();
                Iterator it = pmap.entrySet().iterator();
                while (it.hasNext()) {
                    Map.Entry me = (Map.Entry) it.next();
                    String key = (String) me.getKey();
                    sb.append(sep).append(key).append("=");
                    Object o = me.getValue();
                    if (o instanceof Object[]) {
                        String sep2 = "";
                        Object parms[] = (Object[]) o;
                        for (int i=0; i<parms.length; i++) {
                            sb.append(sep2+parms[i]);
                            sep2=",";
                        }
                    } else {
                        sb.append(""+o);
                    }
                    sep = "&";
                }
                MDC.put("params",sb.toString());
            }
//csc_062405_1_end

            // cater for Enhydra's different behaviour if in a PO
            if (null==target || target.length()==0) {
                target = req.getRequestURI();
                if (logger.isDebugEnabled()) logger.debug("Setting target to request URI: " + target);
            }

            //..trim any path info (based on '/')
            if (target!=null) {
                int spos = target.lastIndexOf("/");
                if (spos>-1) target = target.substring(spos+1);
            }
            //..trim any path info (based on '\')
            if (target!=null) {
                int spos = target.lastIndexOf("\\");
                if (spos>-1) target = target.substring(spos+1);
            }

            //now get the actual name of the event
            String ext = eventBroker.getEventExtension();
            String eventName = target;
            if (eventName.endsWith(ext)) {
                eventName = target.substring(0, target.length()-ext.length());
                if (logger.isDebugEnabled()) logger.debug("Target Event:"+eventName);
            }
            if (mdcLogger.isInfoEnabled()) MDC.put("event", target); //csc_062405_1

            //now get the actual event name
            if (allowEventAliasing) try {
                String fullEventName = eventBroker.matchEventClass(eventName);
                if (fullEventName!=null) {
                    eventName = fullEventName;
                    if (logger.isDebugEnabled()) logger.debug("Fully qualified Event:"+eventName);
                }
            } catch (InvalidClassException e) {}

            //now instantiate an Event (here's where it would make sense to
            //use an Event pool)
            if (logger.isDebugEnabled()) logger.debug("Creating Event");
            try {
                //get the class name for the event
//csc_060903_3                Class cl = Thread.currentThread().getContextClassLoader().loadClass(eventName);
                Class cl = Classes.getClass(eventName); //csc_060903_3

                //get it from the pool if appropriate
                if (useEventPool) try {
                    if (logger.isDebugEnabled()) logger.debug("Checking out event from event pool");
                    event = eventPool.checkoutEvent(cl);
                    eventCameFromPool = true;
                } catch (NoAvailableEventsException e) {
                    if (logger.isDebugEnabled()) logger.debug("No available events in pool");
                }

                //if that didn't work (or we're not getting it from the
                //pool in the first place) create it manually
                if (event==null) {
                    if (logger.isDebugEnabled()) logger.debug("Creating a new event from scratch");
                    event = (BaseEvent) cl.newInstance();    //this requires that events being dispatched have a noargs constructor
                }

            } catch (Exception e) {
                //if we can't map the request to an event, we will just
                //dispatch a generic HttpRequestEvent
//csc_100603_1_start
//                logger.error("Class "+eventName+" not found...Creating a generic HttpRequestEvent");
//                event = new HttpRequestEvent();
                if (RESPOND_WITH_404) {

//csc_010404_1_start
//                    logger.warn("Class "+eventName+" not found...Responding with 404 error");
//                    resp.sendError(HttpServletResponse.SC_NOT_FOUND);
                    if (oresp!=null && dresp!=null) {
                        ObjectRepository lror = ObjectRepository.getObjectRepository(LR_OR, ObjectRepository.THREADSAFE);
                        lror.putState(lrid+"_error", new Integer(HttpServletResponse.SC_NOT_FOUND));
                    } else {
                        logger.warn("Class "+eventName+" not found...Responding with 404 error");
                        resp.sendError(HttpServletResponse.SC_NOT_FOUND);
                    }
                    return;
//csc_010404_1_end
                } else {
                    logger.error("Class "+eventName+" not found...Creating a generic HttpRequestEvent");
                    event = new HttpRequestEvent();
                }
//csc_100603_1_end
            }

            //make sure the incoming event implements HttpRequestEvent...if not,
            //consider it invalid (This is VERY important: it prevents people
            //from hacking the system by directly invoking events that were
            //not intended to be invoked directly from teh outside world)
            if (!(event instanceof HttpRequestEvent)) event = new HttpRequestEvent();







//csc_010404_1_start
            //if we encounter a LongRunning event...
            if (event instanceof LongRunning && !lrOverride) {

                //if we're dealing with a LongRunning event, make sure the response object implements
                //BarracudaServletResponseWrapper (else there's no point continuing)
                if (!(req instanceof BarracudaServletRequestWrapper)) throw new EventException("LongRunning support not enabled - you must use a req wrapper that implements BarracudaServletRequestWrapper");
                if (!(resp instanceof BarracudaServletResponseWrapper)) throw new EventException("LongRunning support not enabled - you must use a resp wrapper that implements BarracudaServletResponseWrapper");

                //if its NOT already running as a long running response (ie. the URL does not include the
                //$lrid=xxxxx param yet), then we are on the first pass: redirect accordingly
                if (lrid==null) {
                    if (logger.isInfoEnabled()) logger.info("Redirecting for CheckLongRunningEvent (pass 1)");
                    LongRunningEventGateway.redirectBrowserToKickoffLongRunningRequest(lrid, req, resp, vc, ourl, oqstr, event);
/*
                    //create a new deferred response and place it in the lror. We also want
                    //to save a copy of the params so we can reconstitute those on subsquent requests
                    ObjectRepository lror = ObjectRepository.getObjectRepository(LR_OR, ObjectRepository.THREADSAFE);
                    lrid = getUID();
                    lror.putState(lrid, new DeferredResponseWrapper(LR_PIPE_SIZE, LR_PIPE_TIMEOUT));
                    lror.putState(lrid+"_params", new HashMap(req.getParameterMap()));
                    lror.putState(lrid+"_orig_url", ourl+(oqstr!=null ? "?"+oqstr : ""));
                    lror.putState(lrid+"_orig_method", req.getMethod());
                    lror.putState(lrid+"_event", event);

                    //now build the check status url
                    ClientSideRedirectException rdException = new ClientSideRedirectException(new CheckLongRunningEvent());
                    String rdURL = URLRewriter.encodeRedirectURL(req, resp, rdException.getRedirectURL()+"?"+LongRunningEventGateway.LONG_ID+"="+lrid);
                    rdURL = ScriptDetector.prepareRedirectURL(rdURL, vc);
                    if (logger.isInfoEnabled()) logger.info("rdURL: "+rdURL);

                    //now build the long running url
                    String lrURL = ourl+(oqstr!=null ? "?"+oqstr : "")+(oqstr!=null ? "&" : "?")+LR_ID+"="+lrid;
                    if (logger.isInfoEnabled()) logger.info("lrURL: "+lrURL);

                    //send the redirect request
                    resp.setContentType("text/html");
                    StringBuffer sb = new StringBuffer(500);
                    sb.append("<html>");
                    sb.append("  <head>");
                    sb.append("    <title>Redirecting...</title>");
                    sb.append("  </head>");
                    sb.append("  <script language=\"javascript\">");
                    sb.append("    var pageFullyLoaded = false;");
                    sb.append("    function handleComplete() {");
                    sb.append("        if (!pageFullyLoaded) {");
                    sb.append("            window.top.location.replace('CancelLongRunningEvent.event?"+LR_ID+"="+lrid+"');");
                    sb.append("            alert('In the future, please press the Cancel button to properly stop a long running process');");
                    sb.append("        }");
                    sb.append("    }");
                    sb.append("  </script>");
                    if (LR_DEBUG==1) {              //allow resize
                        sb.append("  <frameset rows=\"100%,200\" onload=\"handleComplete();\">");
                        sb.append("    <frame src=\""+rdURL+"\">");
                        sb.append("    <frame src=\""+lrURL+"\">");
                        sb.append("  </frameset>");
                    } else if (LR_DEBUG==2) {       //don't show the check frame
                        sb.append("  <frameset rows=\"100%,200\" onload=\"handleComplete();\">");
                        sb.append("    <frame src=\"foo.html\" >");
                        sb.append("    <frame src=\""+lrURL+"\">");
                        sb.append("  </frameset>");
                    } else {                        //normal
                        sb.append("  <frameset rows=\"100%,1\" onload=\"handleComplete();\">");
                        sb.append("    <frame src=\""+rdURL+"\" frameborder=\"0\" noresize>");
                        sb.append("    <frame src=\""+lrURL+"\" frameborder=\"0\" noresize>");
                        sb.append("  </frameset>");
                    }
//                    sb.append("  </frameset>");
                    sb.append("</html>");
                    String content = sb.toString();
                    Writer wr = resp.getWriter();
                    wr.write(content);
                    wr.close();
                    resp.flushBuffer();
*/
                    //once we've done all this, simply return
                    return;

                //if we are actually running generating the long running response (pass 2),
                //set the event in the resp object (it will be needed for progress updates)
                } else {
                    LongRunning lr = (LongRunning) event;
                    lr.reset();
                    dresp.setLongRunning(lr);
                    dresp.setLongRunningThread(Thread.currentThread());
                }
            }
//csc_010404_1_end







            //now set any special paramters
            event.setSource(this);
            event.setHandled(false);

            //now figure out if the event targets any specific listener IDs, and
            //if so add these ids to the event. This info will be used by the
            //dispatcher to send the event to the specified listeners, rather
            //doing a general dispatch to all parties interested in this class of
            //event.
            String[] ids = req.getParameterValues(BaseEvent.EVENT_ID);
            if (logger.isDebugEnabled()) logger.debug("Looking for event listener ids:"+ids);
            if (ids!=null) {
                for (int i=0, max=ids.length; i<max; i++) {
                    String id = ids[i];
                    if (logger.isDebugEnabled()) logger.debug("Target EventID:"+id);
                    if (allowEventAliasing) try {
                        id = eventBroker.matchListenerID(ids[i]);
                        if (logger.isDebugEnabled()) logger.debug("Fully qualified EventID:"+id);
                    } catch (InvalidClassException e) {}
                    event.addListenerID(id);
                }
            }

            //finally, dispatch it to the EventBroker
            if (logger.isDebugEnabled()) logger.debug("Dispatching event:"+event+" to broker:"+eventBroker);
            DispatchQueue eventQueue = getNewDispatchQueueInstance();
            HttpResponseEvent defaultResponseEvent = new HttpResponseEvent();
            defaultResponseEvent.setSource(event);
            eventQueue.addEvent(event);
            context = new DefaultEventContext(eventQueue, vc, this.getServletConfig(), req, resp, defaultResponseEvent);
            context.putState(TARGET_EVENT_NAME, eventName);
            if (null!=externalContextObj) {
                context.putState(EXTERNAL_CONTEXT_OBJ_NAME, externalContextObj);
            }
            if (logger.isInfoEnabled()) smillis = System.currentTimeMillis();
            eventBroker.dispatchEvent(context);
            if (logger.isInfoEnabled()) emillis = System.currentTimeMillis();
            if (logger.isInfoEnabled()) logger.info("Dispatching complete! (handled in "+(emillis-smillis)+" of "+(emillis-bmillis)+" millis)");


        } catch (ClientSideRedirectException re) {
            //save current context to session
            if (context!=null) {
                if (logger.isDebugEnabled()) logger.debug("Saving context");
                context.persistContext(re);
            }

            //if we get a redirect exception, request the browser to redirect accordingly
            String url = URLRewriter.encodeRedirectURL(req, resp, re.getRedirectURL());
            url = ScriptDetector.prepareRedirectURL(url, vc);     //csc_102501.2
            if (logger.isInfoEnabled()) logger.info("ClientSideRedirectException...redirecting to "+url);

//csc_042104_1_start
//            resp.sendRedirect(url);
            if (oresp!=null && dresp!=null && !dresp.isInterrupted()) {
                dresp.sendRedirect(url);
                LongRunningEventGateway.redirectBrowserToGetLongRunningResults(lrid, dresp.getUnderlyingResponse());
            } else {
                resp.sendRedirect(url);
            }    
//csc_042104_1_end

        } catch (EventException e) {
            //csc_010404_1_start
            //if we are dealing with a deferred response, and its been interrupted, just return
            if (logger.isInfoEnabled()) logger.debug("got EventException: "+e, e);
            if (oresp!=null && dresp!=null) {
                if (dresp.isInterrupted()) return;
                ObjectRepository lror = ObjectRepository.getObjectRepository(LR_OR, ObjectRepository.THREADSAFE);
                lror.putState(lrid+"_exception", e);
                return;
            }
            //csc_010404_1_send

            //if we get an EventException, handle it
            handleEventException(e, req, resp);

        //csc_010404_1_start
        } catch (ServletException e) {
            //if we are dealing with a deferred response, and its been interrupted, just return
            if (logger.isInfoEnabled()) logger.debug("got ServletException: "+e, e);
            if (oresp!=null && dresp!=null) {
                if (dresp.isInterrupted()) return;
                ObjectRepository lror = ObjectRepository.getObjectRepository(LR_OR, ObjectRepository.THREADSAFE);
                lror.putState(lrid+"_exception", e);
                return;
            } else {
                throw e;
            }

        } catch (IOException e) {
            //if we are dealing with a deferred response, and its been interrupted, just return
            if (logger.isInfoEnabled()) logger.debug("got IOException: "+e, e);
            if (oresp!=null && dresp!=null) {
                if (dresp.isInterrupted()) return;
                ObjectRepository lror = ObjectRepository.getObjectRepository(LR_OR, ObjectRepository.THREADSAFE);
                lror.putState(lrid+"_exception", e);
                return;
            } else {
                throw e;
            }

        } catch (RuntimeException e) {
            //if we are dealing with a deferred response, and its been interrupted, just return
            if (logger.isInfoEnabled()) logger.debug("got RuntimeException: "+e, e);
            if (oresp!=null && dresp!=null) {
                if (dresp.isInterrupted()) return;
                ObjectRepository lror = ObjectRepository.getObjectRepository(LR_OR, ObjectRepository.THREADSAFE);
                lror.putState(lrid+"_exception", e);
                return;
            } else {
                throw e;
            }
        //csc_010404_1_send

        } finally {
            //csc_010404_1_start
            //now, on a final note - if we redirected the response to lror, once we are all finished,
            //write a response to the original resp object redirecting the client page back to the result
            if (oresp!=null && !dresp.isInterrupted()) {
                if (logger.isInfoEnabled()) logger.info("Redirecting client to get results from lror (finish pass 2)");

                //mark the resp as committed
//                dresp.setCommitted(true);

                //reset the response
                resp = oresp;

/*
                //request the browser to redirect accordingly
                ObjectRepository lror = ObjectRepository.getObjectRepository(LR_OR, ObjectRepository.THREADSAFE);
                String rdURL = (String) lror.getState(lrid+"_orig_url");
                lror.removeState(lrid+"_orig_url");
                String sep = (rdURL.indexOf("?")>-1 ? "&" : "?");
                rdURL = rdURL+sep+LR_ID+"="+lrid;
                if (logger.isInfoEnabled()) logger.info("redirecting to "+rdURL);
                resp.setContentType("text/html");
                StringBuffer sb = new StringBuffer(500);
                sb.append("<html>");
                sb.append("  <head>");
                sb.append("    <title>Redirecting...</title>");
                sb.append("    <script src=\"xlib/org/barracudamvc/core/scripts/FormControl.js\" type=\"text/javascript\"></script>");
                sb.append("  </head>");
                sb.append("  <body onload=\"window.top.pageFullyLoaded=true; window.top.location.replace('"+rdURL+"');\">");
                sb.append("    <div style=\"text-align: center\"><a href=\""+rdURL+"\" target=\"_top\">Click here</a> if your browser does not automatically redirect...</div>");
                sb.append("  </body>");
                sb.append("</html>");
                String content = sb.toString();
                Writer wr = resp.getWriter();
                wr.write(content);
                wr.close();
                resp.flushBuffer();
*/
                //request the browser to redirect accordingly
//                LongRunningEventGateway.redirectBrowserToGetLongRunningResults(lrid, resp);
            }
            //csc_010404_1_end




            //make sure we ALWAYS release the event
            if (event!=null && eventCameFromPool) {
                if (logger.isDebugEnabled()) logger.debug("Releasing event");
                eventPool.releaseEvent(event);
                if (logger.isDebugEnabled()) logger.debug("Released!");
            }

            //clean up the log4j NDC, MDC stacks
            NDC.remove();
            MDC.remove("event");    //csc_062405_1
            MDC.remove("params");   //csc_062405_1

            //make sure we always clean up session/local repository stuff
            ObjectRepository.removeLocalRepository();                                       //csc_022101.1
//System.out.println("here88: name:"+logger.getName()+" level:"+logger.getLevel()+" pri:"+logger.getPriority()+" addtv:"+logger.getAdditivity()+" chpri:"+logger.getChainedPriority()+" catg:"+logger.getCurrentCategories()+" dfhier:"+logger.getDefaultHierarchy()+" efflvl:"+logger.getEffectiveLevel()+" hier:"+logger.getHierarchy()+" parent:"+logger.getParent()+" root:"+logger.getRoot());
//System.out.println("here99: name:"+orLogger.getName()+" level:"+orLogger.getLevel()+" pri:"+orLogger.getPriority()+" addtv:"+orLogger.getAdditivity()+" chpri:"+orLogger.getChainedPriority()+" catg:"+orLogger.getCurrentCategories()+" dfhier:"+orLogger.getDefaultHierarchy()+" efflvl:"+orLogger.getEffectiveLevel()+" hier:"+orLogger.getHierarchy()+" parent:"+orLogger.getParent()+" root:"+orLogger.getRoot());
            if (orLogger.isDebugEnabled() && ourl.indexOf("CheckLongRunningEvent")<0) ObjectRepository.printStackTrace("AFT: "+ourl+(oqstr!=null ? "?"+oqstr : ""));
            ObjectRepository.removeSessionRepository();                                     //csc_022101.1
            if (logger.isInfoEnabled()) logger.info("Finished handling incoming HTTP request in "+this);
        }
    }
    
    /**
     * <p>Handle an EventException. Basically, this is where we handle the
     * really bad, unexpected type of event exceptions. Generally, as you code,
     * if you want to interrupt the dispatch and fire a new event, you should
     * throw an InterruptDispatchException. Only throw EventExceptions in
     * truly exceptional circumstances.
     *
     * @param e the EventException to handle
     * @param req the servlet request
     * @param resp the servlet response
     * @throws ServletException
     * @throws IOException
     */
    public void handleEventException (EventException e, HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        //first see what the base exception is...if it's a servlet exception
        //or an IOException, rethrow it...
        Exception rootException = NestableException.getRootException(e);
        if (rootException instanceof ServletException) throw (ServletException) rootException;
        if (rootException instanceof IOException) throw (IOException) rootException;

        //if we get an EventException, log it and print the base exception
        resp.setContentType("text/html");
        PrintWriter out = resp.getWriter();
        ExceptionUtil.logExceptionAsHTML(out, e, req);
        logger.warn("Unexpected event exception: ", e);
        if (rootException!=e) {
            logger.warn("Root Exception...", rootException);
        }
        ApplicationGateway.logRequestDetails(logger, Level.WARN);    //csc_031005_1
    }

    //csc_031005_1 - added
    /**
     * This method is used to log param details associated with the current request. It is called
     * automatically when Barracuda hits a serious exception; you may wish to call it manually in
     * your own code that extends BaseEventListener
     */    
    public static void logRequestDetails(Logger l, Level level) {
        //get the request info if present
        StringBuffer sb = new StringBuffer(1000);
        String newline = "\n";        
        ObjectRepository lor = ObjectRepository.getLocalRepository();
        HttpServletRequest req = (HttpServletRequest) lor.getState(ApplicationGateway.HTTP_SERVLET_REQUEST);
        if (req!=null) {
            sb.append(newline+"...(http uri)     "+req.getRequestURI());
            sb.append(newline+"...(http params)  ");
            Map pmap = new TreeMap();
            Enumeration enum3 = req.getParameterNames();
            while (enum3.hasMoreElements()) {
                String key =(String) enum3.nextElement();
                String vals[] = req.getParameterValues(key);
                pmap.put(key, vals);
            }
            Iterator it = pmap.entrySet().iterator();
            while (it.hasNext()) {
                Map.Entry me = (Map.Entry) it.next();
                String key =(String)  me.getKey();
                String vals[] = (String[]) me.getValue();                
                for (int i=0, max=vals.length; i<max; i++) {
                    String s = vals[i];     
                    if (key.equalsIgnoreCase("password") || key.equalsIgnoreCase("pwd")) s = "********";        
                    sb.append(newline+"      key:"+key+" value:"+s);
                }
            }
        } else {
            sb.append(newline+"...(http uri)     n/a");
            sb.append(newline+"...(http params)  n/a");
        }
        l.log(level, "Logging additional request details:"+sb.toString());     
    }
    
    
    
    

    //-------------------- ApplicationGateway --------------------
    /**
     * <p>Specify event gateways. You can call this method with
     * as many different gateways as you like. When the Application
     * Gateway initializes, it will load them all. When the gateway
     * is destroyed, they will be deregistered.
     *
     * @param gateway an event gateway we'd like to have registered
     */
    public final void specifyEventGateways(EventGateway gateway) {
        if (gateways==null) gateways = new ArrayList();
        gateways.add(gateway);
    }

    /**
     * <p>Provide a reference to the event broker.
     *
     * @return a reference to the master event broker
     */
    public final EventBroker getEventBroker() {
        if (masterEventBroker==null) {
            masterEventBroker = getNewEventBrokerInstance();

            //this ensures that the App-gateway can successfully route
            //generic ActionEvents (if we ever add other "well-known"
            //generic events we should pre-register them here as well
            //so that they can be used without any special registration
            //process)
            try {masterEventBroker.addEventAlias(ActionEvent.class);}
            catch (InvalidClassException e) {}
        }
        return masterEventBroker;
    }

    /**
     * <p>Provide a reference to the event pool. May return
     * null if we're not using event pooling.
     *
     * @return a reference to the master event pool
     */
    public final EventPool getEventPool() {
//csc_060903_1        if (USE_EVENT_POOLING && masterEventPool==null) masterEventPool = getNewEventPoolInstance();
        if (USE_EVENT_POOLING && masterEventPool==null) masterEventPool = (EventPool) Classes.newInstance(A_Classes.DEFAULT_EVENT_POOL);    //csc_060903_1
        if (!USE_EVENT_POOLING) masterEventPool = null;
        return masterEventPool;
    }

    //dbr_021602 - added
    protected HttpServletRequest wrapRequest(HttpServletRequest req) {
        if (REQUEST_WRAPPER!=null) {
            return REQUEST_WRAPPER.wrap(req);
        } else {
            return new DefaultServletRequestWrapper(req);
        }
    }

    protected HttpServletResponse wrapResponse(HttpServletResponse resp) {
        if (RESPONSE_WRAPPER!=null) {
            return RESPONSE_WRAPPER.wrap(resp);
        } else {
            return new DefaultServletResponseWrapper(resp);
        }
    }







    //-------------------- EventGateway --------------------------
    /**
     * <p>Set the parent gateway. Null indicates its the root.
     * By definition, an ApplicationGateway is always the root.
     *
     * @param eg the parent event gateway for this gateway
     */
    public final void setParent(EventGateway eg) {
        eventGateway.setParent(null);
    }

    /**
     * <p>Get the parent gateway. Returns null if it's the root.
     *
     * @return the parent event gateway
     */
    public final EventGateway getParent() {
        return eventGateway.getParent();
    }

    /**
     * <p>Add an event gateway to this one
     *
     * @param eg the event gateway to be added
     */
    public final void add(EventGateway eg) {
        eventGateway.add(eg);
        eg.setParent(this);    //do this so the parent points at the ApplicationGateway, not the EventGateway we are using to actually store the values
    }

    /**
     * <p>Remove an event gateway from this one
     *
     * @param eg the event gateway to be removed
     */
    public final void remove(EventGateway eg) {
        eventGateway.remove(eg);
    }

    /**
     * Get a list of child gateways. The list returned is a copy of the
     * underlying child gateway list.
     *
     * @return a list of child gateways
     */
    public List getChildren() {
        return eventGateway.getChildren();
    }

    /**
     * <p>Ask all interested parties to register with
     * the EventBroker
     *
     * @param eb the event broker to register with
     */
    public final void register(EventBroker eb) {
        eventGateway.register(eb);
    }

    /**
     * <p>Ask all interested parties to de-register with
     * the EventBroker
     *
     * @param eb the event broker to de-register with
     */
    public final void deregister(EventBroker eb) {
        eventGateway.deregister(eb);
    }

    /**
     * Register any local interests in the EventBroker
     *
     * @param eb the event broker this gateway should use to
     *        register for local events
     */
    public final void registerLocalEventInterests(EventBroker eb) {
        eventGateway.registerLocalEventInterests(eb);
    }

    /**
     * Deregister any local interests in the EventBroker
     *
     * @param eb the event broker this gateway should use to
     *        de-register for local events
     */
    public final void deregisterLocalEventInterests(EventBroker eb) {
        eventGateway.deregisterLocalEventInterests(eb);
    }

    /**
     * Register any local event aliases in the EventBroker
     *
     * @param eb the event broker this gateway should use to
     *        register aliases for local events
     */
    public final void registerLocalEventAliases(EventBroker eb) {
        eventGateway.registerLocalEventAliases(eb);
    }

    /**
     * Rather than overriding the registerLocalEventInterests
     * method, you can just invoke this method instead for each
     * interest you'd like to register. The gateway will keep track
     * of all factories specified, and register/deregister when
     * appropriate (so you don't have to worry about it). Notice that
     * this method registers just the listener id (not for a specific
     * class of event).
     *
     * The only real reason for using the registerLocalEventInterests
     * method would be if you actually need access to the EventBroker.
     *
     * Note that if the event class is not an instance of BaseEvent, the
     * request is just silently ignored (unlike the event broker, which
     * throws an exception).
     *
     * @param factory the factory we wish to register with the event broker
     */
    public final void specifyLocalEventInterests(ListenerFactory factory) {
        eventGateway.specifyLocalEventInterests(factory);
    }

    /**
     * Rather than overriding the registerLocalEventInterests
     * method, you can just invoke this method instead for each
     * interest you'd like to register. The gateway will keep track
     * of all factories specified, and register/deregister when
     * appropriate (so you don't have to worry about it).
     *
     * The only real reason for using the registerLocalEventInterests
     * method would be if you actually need access to the EventBroker.
     *
     * Note that if the event class is not an instance of BaseEvent, the
     * request is just silently ignored (unlike the event broker, which
     * throws an exception).
     *
     * @param factory the factory we wish to register with the event broker
     * @param event the class of events we are interested in
     */
    public final void specifyLocalEventInterests(ListenerFactory factory, Class event) {
        eventGateway.specifyLocalEventInterests(factory, event);
    }

    /**
     * Rather than overriding the registerLocalEventAliases
     * method, you can just invoke this method instead for type
     * of event you want to manually alias.
     *
     * The only real reason for using the registerLocalEventAliases
     * method would be if you actually need access to the EventBroker.
     *
     * Note that if the event class is not an instance of BaseEvent, the
     * request is just silently ignored (unlike the event broker, which
     * throws an exception).
     *
     * @param event the class of events we are interested in registering
     *        aliases for
     */
    public final void specifyLocalEventAliases(Class event) {
        eventGateway.specifyLocalEventAliases(event);
    }






    //-------------------- HTTPServlet ---------------------------
    /**
     * <p>By default the GET request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param resp the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
//csc_102201.1_start
//        handleDefault(new org.barracudamvc.core.helper.servlet.HttpServletRequestWrapper(wrap(req)), resp);
        boolean handled = ScriptDetector.checkClientReq(req, resp);
        //dbr_021602_start.2
        if (!handled) handleDefault(wrapRequest(req), wrapResponse(resp));
        //dbr_021602_end.2
//csc_102201.1_end
    }

    /**
     * <p>By default the POST request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param resp the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(wrapRequest(req), wrapResponse(resp));
    }

    /**
     * <p>By default the OPTIONS request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param resp the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doOptions(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(wrapRequest(req), wrapResponse(resp));
    }

    /**
     * <p>By default the DELETE request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param resp the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doDelete(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(wrapRequest(req), wrapResponse(resp));
    }

    /**
     * <p>By default the PUT request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param resp the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doPut(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(wrapRequest(req), wrapResponse(resp));
    }

    /**
     * <p>By default the TRACE request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param resp the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doTrace(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(wrapRequest(req), wrapResponse(resp));
    }






    //-------------------- Servlet -------------------------------
    /**
     * <p>Here's where we initialize the servlet.
     */
    public void init() throws ServletException {

        try {
            logger.info("initializing ApplicationGateway servlet");


//csc_060903_3_start - why are we even doing this section??? This seems redundant to me, like its
//something that should be happening via the ObjectRepositoryAssembler, NOT ApplicationGateway
//parameters...Jake seems to think so too, so I'm commenting it out for now
/*
            //first and foremost, assign any debug value constants
            logger.debug("checking for DebugValues");
            Enumeration enum = this.getServletConfig().getInitParameterNames();
            while (enum.hasMoreElements()) {
                String param = enum.nextElement().toString();
                if (param.startsWith(DEBUG_VALUE)) {
                    String value = this.getServletConfig().getInitParameter(param);
                    logger.debug(param+" = "+value);

                    //split it into parts
                    try  {
                        int pos1 = value.lastIndexOf(".");
                        int pos2 = value.indexOf("=", pos1);
                        String clname = value.substring(0,pos1);
                        String var = value.substring(pos1+1, pos2);
                        String val = value.substring(pos2+1);
                        logger.debug("Class:"+clname+" Variable:"+var+" Value:"+val);

                        //figure out what value we're trying to set
//csc_060903_3                        Class cl = Thread.currentThread().getContextClassLoader().loadClass(clname);
                        Class cl = Classes.getClass(clname);    //csc_060903_3

                        Field field = cl.getDeclaredField(var);
                        Object origval = field.get(null);
                        Class fieldType = field.getType();
                        Object targetval = null;
                        //...integer
                        if (fieldType==Integer.class || fieldType==int.class) {
                            targetval = new Integer(val);
                        //...boolean
                        } else if (fieldType==Boolean.class || fieldType==boolean.class) {
                            targetval = new Boolean(val);
                        //...float
                        } else if (fieldType==Float.class || fieldType==float.class) {
                            targetval = new Float(val);
                        //...double
                        } else if (fieldType==Double.class || fieldType==double.class) {
                            targetval = new Double(val);
                        //...long
                        } else if (fieldType==Long.class || fieldType==long.class) {
                            targetval = new Long(val);
                        //...short
                        } else if (fieldType==Short.class || fieldType==short.class) {
                            targetval = new Short(val);
                        //...String (def)
                        } else {
                            targetval = val;
                        }

                        //actually try setting it
                        field.set(null, targetval);
                        Object newval = field.get(null);
                        logger.debug("set "+clname+"."+var+" from "+origval+" to "+newval);

                    } catch (Exception e) {
                        logger.error("err "+e+" setting value:"+value);
                        continue;
                    }
                }
            }
*/
//csc_060903_3_end

//csc_060903_2_start - move down below so that EventPool could be scripted via the application assembler
            //start by getting a reference to the EventBroker. Also request the
            //EventPool, just to make sure they're both instantiated
//            logger.debug("creating EventBroker");
//            EventBroker eb = this.getEventBroker();
//            getEventPool();
//csc_060903_2_end

            //before performing automated gateway assembly, first manually add in a reference to
            //LongRunningEventGateway (we do this so its always available).
            this.add(new LongRunningEventGateway());

            //perform any automated assembly if necessary
            String assemblerName = this.getServletConfig().getInitParameter(APPLICATION_ASSEMBLER);
            String descriptor = this.getServletConfig().getInitParameter(ASSEMBLY_DESCRIPTOR);
            if (assemblerName!=null && descriptor!=null) try {
                logger.debug("performing automated assembly");
                String parser = this.getServletConfig().getInitParameter(SAX_PARSER);
//csc_060903_1                ApplicationAssembler assembler = (ApplicationAssembler) Thread.currentThread().getContextClassLoader().loadClass(assemblerName).newInstance();
                ApplicationAssembler assembler = (ApplicationAssembler) Classes.newInstance(assemblerName);  //csc_060903_1
                if (parser!=null) assembler.assemble(this, this.getServletConfig(), descriptor, parser);
                else assembler.assemble(this, this.getServletConfig(), descriptor);
                logger.debug("automated assembly complete!");
            } catch (Exception e) {
                throw new RuntimeException ("Error invoking assembler:"+e);
            }

//csc_060903_2_start
            //start by getting a reference to the EventBroker. Also request the
            //EventPool, just to make sure they're both instantiated. We do this here
            //primarily so that if there is a problem instantiating these, we find
            //out about it immediately on startup, rather than waiting until a request
            //actually comes through.
            logger.debug("creating EventBroker & EventPool");
            EventBroker eb = this.getEventBroker();
            getEventPool();
//csc_060903_2_end

            //allow for local initialization
            logger.debug("initialize local");
            initializeLocal();

            //handle any gateways which were specified
            if (gateways!=null) {
                logger.debug("handle any local gateways");
                Iterator it = gateways.iterator();
                while (it.hasNext()) {
                    EventGateway gateway = (EventGateway) it.next();
                    logger.debug("adding specified event gateway:"+gateway);
                    this.add(gateway);
                }
            }

            //now register the event gateway (note that we have to set the value to
            //false when registering to ensure that all possible combinations get
            //registered. If we don't do this we run into problems later)
            logger.debug("invoke register");
            boolean aliasVal = DefaultBaseEvent.USE_ID_ALIASES;
            DefaultBaseEvent.USE_ID_ALIASES = false;
            register(eb);
            DefaultBaseEvent.USE_ID_ALIASES = aliasVal;

            //show the app structure
/*
//csc_060903_3 - removed because it is calling methods that are not part of the EventGateway interface (and its not really used anyway)
            if (SHOW_APP_STRUCTURE) {
                logger.debug("Printing Stack Trace...");
                eventGateway.printStackTrace(0, System.out);
            }
*/
            virgin = true;
            if (logger.isInfoEnabled()) logger.info("Initialization Complete!");
        } catch (RuntimeException e) {
            logger.fatal("ALERT!!!!! Fatal err initializing servlet:", e);
            throw e;
        }
    }

    /**
     * <p>Here's where we destroy the servlet.
     */
    public void destroy() {
        try {
            logger.info("destroying servlet");

            //start by getting a reference to the EventBroker
            EventBroker eb = this.getEventBroker();

            //now deregister the event gateway
            boolean aliasVal = DefaultBaseEvent.USE_ID_ALIASES;
            DefaultBaseEvent.USE_ID_ALIASES = false;
            deregister(eb);
            DefaultBaseEvent.USE_ID_ALIASES = aliasVal;

            //finally allow for any local destruction
            destroyLocal();

            //remove any gateways which were specified
            if (gateways!=null) {
                Iterator it = gateways.iterator();
                while (it.hasNext()) {
                    EventGateway gateway = (EventGateway) it.next();
                    this.remove(gateway);
                }
            }

            //lb_032801_start - Patch submitted by Larry Brasfield
            //[larry.brasfield@theplatform.com] so JVM can exit once
            //the servlet's destroy() method is called.  The problem
            //is that the event pool cleanup thread will not stop. The
            //following code corrects this. You can find related changes
            //by searching for lb_032801
            //
            //Allow finalizer to get rid of this pool object.
            //That way, a shutdown can actually be effected.
            if (masterEventPool!=null) {
                masterEventPool.shutdown();
                masterEventPool = null;
            }
            masterEventBroker = null;
            System.gc();
            System.runFinalization();
            //lb_032801_end

        } catch (RuntimeException e) {
            logger.fatal("ALERT!!!!! Fatal err destroying servlet:", e);
            throw e;
        }
    }

}
