/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: LongRunning.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.core.event;

import java.util.List;

import org.barracudamvc.plankton.data.StateMap;

/**
 * This interface indicates an EventHandler takes a long time to run. As such,
 * Barracuda will automatically send a response back to the browser giving the 
 * user the opportunity to a) see that its going to take a while and b) interrupt
 * the process if they so choose.
 *
 * @author christianc@granitepeaks.com
 * @since 010404_1
 */
public interface LongRunning {

    /**
     * Specify the redirect event to be fired if the LongRunning process is cancelled
     */
    public void setRedirectEvent(BaseEvent be);
    
    /**
     * get the redirect event for this LongRunning process. Typically called by LongRunningEventGateway.
     */
    public BaseEvent getRedirectEvent();
    
    /**
     * set the refresh rate (in secs). -1 indicates the DEFAULT_REFRESH_RATE will be used
     */
    public void setRefreshRate(int secs);
    
    /**
     * get the refresh rate (in secs). If the refresh rate has not been set, 
     * DEFAULT_REFRESH_RATE will be returned.
     */
    public int getRefreshRate();

    /**
     * set ETA (in secs). -1 indicates the DEFAULT_ETA will be used
     */
    public void setETA(int secs);
    
    /**
     * get ETA (in secs). If the ETA has not been set, DEFAULT_ETA will be returned.
     */
    public int getETA();
    
    /**
     * set elapsed (in secs). This can be called by the developer's event handler code if desired 
     * to manually specify how much time has elapsed. If you don't call this method, elapsed time
     * be determined dynamically by comparing the amount of time elapsed from object creation.
     */
    public void setElapsed(int secs);
    
    /**
     * get elapsed (in secs). If a elapsed time has not been programatically specified, 
     * it will be calulated dynamically. Typically called by LongRunningEventGateway.
     */
    public int getElapsed();

    /**
     * set the percent complete (int between 0 and 100). This can be called by the developer's event
     * handler code if desired to manually specify completion percent. If you don't call this method,
     * percent completed will be determined dynamically by comparing the amount of time elapsed with ETA.
     */
    public void setPercentComplete(int percent);
    
    /**
     * get the percent complete (int between 0 and 100). If a percentage complete has not been 
     * programatically specified, it will be calulated dynamically. Typically called by 
     * LongRunningEventGateway.
     */
    public int getPercentComplete();

    /**
     * get the statemap (may be used for storing key/val info, which can then
     * be retrieved from the template). Typically called by the developer's event handling code to 
     * set some kind of value, and then by LongRunningEventGateway to retrieve that value when 
     * requested by the template.
     */
    public StateMap getStateMap();

    /**
     * provide a List of additional BTemplate models to make available during to the 
     * LongRunningEventGateway template. This method would be called by the developers event 
     * handler code to provide additional models for the template to access.
     */
    public void setAdditionalModels(List imodels);

    /**
     * get any additional BTemplate models. Typically called by LongRunningEventGateway.
     */
    public List getAdditionalModels();

    /**
     * provide a different Template class instead of the default Barracuda template, This
     * will typically be used by developers who want to override L&F of the progress screen.
     */
    public void setTemplateClass(Class cl);

    /**
     * get the custom template class. Typically called by LongRunningEventGateway.
     */
    public Class getTemplateClass();
    
    /**
     * resets the start time. Typically called by ApplicationGateway prior to dispatching.
     */
    public void reset();
    
}
