/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: DeferredResponseWrapper.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.core.helper.servlet;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.Reader;
import java.io.Writer;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.TreeMap;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.barracudamvc.core.event.LongRunning;
import org.barracudamvc.core.event.events.LongRunningEventGateway;
import org.barracudamvc.plankton.io.BetterPipedInputStream;
import org.barracudamvc.plankton.io.BetterPipedOutputStream;
import org.barracudamvc.plankton.io.BetterPipedReader;
import org.barracudamvc.plankton.io.BetterPipedWriter;
import org.barracudamvc.plankton.io.Pausable;

//csc_010404_1 - created
/**
 * <p>This class acts as a thin wrapper around a ServletResponse. By using this wrapper, 
 * the response is actually "absorbed" or "deferred", so that it can be manually extracted
 * later.
 */
public class DeferredResponseWrapper implements BarracudaServletResponseWrapper, Pausable {

    protected static final Logger logger = Logger.getLogger(DeferredResponseWrapper.class.getName());

    static final String eol = System.getProperty("line.separator");

    protected final SimpleDateFormat sdf = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss zzz", Locale.US);  //from org.apache.catalina.connector.HttpResponseBase
    HttpServletResponse uresp = null;
    String lrid = null;
    LongRunning lr = null;
    Thread th = null;
    BetterPipedReader pr = null;
    BetterPipedWriter pw = null;
    BetterPipedInputStream pi = null;
    BetterPipedOutputStream po = null;
    Map headers = null;
    List cookies = null;
    boolean gotStream = false;
    boolean gotWriter = false;
    protected int pipeSize = -1;
    protected int pipeTimeout = -1;
    

    int bufferSize = -1;
    int contentLength = -1;
    String contentType = "text/html";
    Locale locale = null;
    int sc = HttpServletResponse.SC_OK;
    String scmsg = getStatusMessage(sc);
    int err = -1;
    String errmsg = null;
    String redirectLoc = null;
    boolean committed = false;
    boolean interrupted = false;
    boolean paused = false;
    Object sync = new Object();
    
    /**
     * Create a DeferredResponseWrapper
     */
    public DeferredResponseWrapper() {
    }

    /**
     * Create a DeferredResponseWrapper, specifying a pipe size and timeout
     */
    public DeferredResponseWrapper(int ipipeSize, int ipipeTimeout) {
        setPipeSize(ipipeSize);
        setPipeTimeout(ipipeTimeout);
    }

    protected void _resetVars() {    
        pr = null;
        pw = null;
        gotStream = false;
        gotWriter = false;
        headers = null;
        cookies = null;
        bufferSize = -1;
        contentLength = -1;
        contentType = "text/html";
        sc = HttpServletResponse.SC_OK;
        scmsg = getStatusMessage(sc);
        err = -1;
        errmsg = null;
        redirectLoc = null;
        committed = false;
        interrupted = false;
        paused = false;
    }

    public boolean isReady() throws IOException {
        boolean isReady = false;
        try {
            isReady = (pr!=null && pr.ready()) || (pi!=null && pi.ready());
        } catch (IOException e) {
            logger.warn("Unexpected IOException checking isReady: "+e, e);
        }
        return (err>-1 || redirectLoc!=null || isReady);
    }

    /**
     * Go ahead and actually write the response
     */
    public void actuallySendResponse(HttpServletResponse resp) throws IOException {
        if (logger.isInfoEnabled()) logger.info("actuallySendResponse()");
        wrLoop: while(true) {
            if (isReady()) {
//csc_042104_1_start
                //else see if we need to send a redirect
                if (redirectLoc!=null) {
                    if (logger.isInfoEnabled()) logger.info("...sending redirect: "+redirectLoc);
                    resp.sendRedirect(redirectLoc);
                    break wrLoop;
                }
//csc_042104_1_end

                //adjust the response
                if (logger.isInfoEnabled()) logger.info("...setting buffer size, content type, etc");
                if (bufferSize>-1) resp.setBufferSize(bufferSize);
                if (contentLength>-1) resp.setBufferSize(contentLength);
                if (contentType!=null) resp.setContentType(contentType);
                if (locale!=null) resp.setLocale(locale);
                
                //add in any headers
                if (headers!=null) {
                    if (logger.isInfoEnabled()) logger.info("...setting headers");
                    Iterator it = headers.entrySet().iterator();
                    while (it.hasNext()) {
                        Map.Entry me = (Map.Entry) it.next();
                        String key = (String) me.getKey();
                        Iterator it2 = ((List) me.getValue()).iterator();
                        while (it2.hasNext()) {
                            resp.addHeader(key, ""+it2.next());
                        }
                    }
                }

                //add in any cookies
                if (cookies!=null) {
                    if (logger.isInfoEnabled()) logger.info("...setting cookies");
                    Iterator it = cookies.iterator();
                    while (it.hasNext()) {
                        resp.addCookie((Cookie) it.next());
                    }
                }

                //set the error code if any
                if (err>-1) {
                    if (logger.isInfoEnabled()) logger.info("...sending err code: "+err+" errmsg:"+errmsg);
                    resp.sendError(err, errmsg);
        
//csc_042104_1_start
/*
                //else see if we need to send a redirect
                } else if (redirectLoc!=null) {
                    if (logger.isInfoEnabled()) logger.info("...sending redirect: "+redirectLoc);
                    resp.sendRedirect(redirectLoc);
*/
//csc_042104_1_end
        
                //else actually write a response
                } else {
                    //finally, set the status
                    resp.setStatus(sc);

                    //then write the response and be done
                    if (logger.isInfoEnabled()) logger.info("...writing response...");
                    if (gotWriter) {
                        Writer realwr = resp.getWriter();
                        int offset = 0;
                        int len = 4096;
                        char[] data = new char[len];
                        while (true) {
                            int cnt = pr.read(data, 0, len);
                            if (cnt==-1) break;
                            realwr.write(data, 0, cnt);
                            resp.flushBuffer();
                            if (logger.isDebugEnabled()) logger.debug("......wrote "+cnt+" chars of data to realwr:"+realwr);
                            offset+=cnt;
                        }
                        if (logger.isInfoEnabled()) logger.info("...finished writing response!");
                        realwr.close();
                        pr.close();
                        bufferSize = offset;
                        break wrLoop;
                    } else {
                        OutputStream realout = resp.getOutputStream();
                        int offset = 0;
                        int len = 4096;
                        byte[] data = new byte[len];
                        while (true) {
                            int cnt = pi.read(data, 0, len);
                            if (cnt==-1) break;
                            realout.write(data, 0, cnt);
                            resp.flushBuffer();
                            if (logger.isDebugEnabled()) logger.debug("......wrote "+cnt+" chars of data to realout:"+realout);
                            offset+=cnt;
                        }
                        if (logger.isInfoEnabled()) logger.info("...finished writing response!");
                        realout.close();
                        pi.close();
                        bufferSize = offset;
                        break wrLoop;
                    }
                }
            } else {
                try {
                    Thread.sleep(300);
                    Thread.yield();
                } catch (InterruptedException e) {
                    logger.info("Got unexpected InterruptedException: "+e, e);
                    break wrLoop;
                }
            }
        }
        if (logger.isInfoEnabled()) logger.info("actuallySendResponse() - done!");
    }

    public void setUnderlyingResponse(HttpServletResponse iuresp, String ilrid) {
        uresp = iuresp;
        lrid = ilrid;
        locale = iuresp.getLocale();
    }    
    
    //csc_042104_1 - added
    public HttpServletResponse getUnderlyingResponse() {
        return uresp;
    }    
    
    public void setLongRunning(LongRunning ilr) {
        lr = ilr;
    }
    
    public LongRunning getLongRunning() {
        return lr;
    }
    
    public void setLongRunningThread(Thread ith) {
        th = ith;
    }
    
    public Thread getLongRunningThread() {
        return th;
    }
    
    public void setPipeSize(int ipipeSize) {
        pipeSize = ipipeSize;
    }
    
    public void setPipeTimeout(int ipipeTimeout) {
        pipeTimeout = ipipeTimeout;
    }
    
    public void interrupt() {
        synchronized (sync) {
            if (!committed) {
                interrupted = true;
                th.interrupt();
            }
        }
    }
    
    public boolean isInterrupted() {
        return interrupted;
    }
    
    public void pause() {
        if (logger.isInfoEnabled()) logger.info("pausing dresp");
        paused = true;
    }

    public void resume() {
        if (logger.isInfoEnabled()) logger.info("resuming dresp");
        paused = false;
    }

    public boolean isPaused() {
        return paused;
    }

    //--------------- ServletResponseWrapper ---------------------
    /**
     * Forces any content in the buffer to be written to the client. This implementation
     * doesn't do aything.
     */
    public void flushBuffer() {
        return;
    }

    /**
     * Returns the actual buffer size used for the response.
     */
    public int getBufferSize() {
        return bufferSize;
    }

    /**
     * Returns the name of the charset used for the MIME body sent in this response.
     * This implementation is hardcoded to return "ISO-8859-1"
     */
    public String getCharacterEncoding() {
        return "ISO-8859-1";
    }

    /**
     * Returns the locale assigned to the response.
     */
    public Locale getLocale() {
        return locale;
    }

    /**
     * Return the servlet output stream associated with this Response.
     *
     * @exception IllegalStateException if <code>getWriter</code> has
     *  already been called for this response, or if the response wrapper
     *  writer has been overridden (in that case you ust use <code>getWriter</code>)
     * @exception IOException if an input/output error occurs
     */
    public ServletOutputStream getOutputStream() throws IOException {
        try {
            if (gotWriter) throw new IllegalStateException("getPrintWriter() has already been called");
            if (gotStream) throw new IllegalStateException("getOutputStream() has already been called");
            if (logger.isInfoEnabled()) logger.info("getOutputStream()");
            po = new BetterPipedOutputStream();
            pi = new CustomizedBetterPipedInputStream();
            pi.setPausable(this);
            if (pipeSize>0) pi.setPipeSize(pipeSize);
            if (pipeTimeout>0) pi.setPipeTimeout(pipeTimeout);
            pi.connect(po);
            return po;
        } finally {
            gotStream = true;
        }
    }

    /**
     * Return the print writer associated with this Response (this method may
     * be called multiple times).
     *
     * @exception IllegalStateException if <code>getOutputStream</code> has
     *  already been called for this response
     * @exception IOException if an input/output error occurs
     */
    public PrintWriter getWriter() throws IOException {
        try {
            if (gotWriter) throw new IllegalStateException("getPrintWriter() has already been called");
            if (gotStream) throw new IllegalStateException("getOutputStream() has already been called");
            if (logger.isInfoEnabled()) logger.info("getWriter()");
            pw = new BetterPipedWriter();
            pr = new CustomizedBetterPipedReader();
            pr.setPausable(this);
            if (pipeSize>0) pr.setPipeSize(pipeSize);
            if (pipeTimeout>0) pr.setPipeTimeout(pipeTimeout);
            pr.connect(pw);
            return new PrintWriter(pw);
        } finally {
            gotWriter = true;
        }
    }
    
    class CustomizedBetterPipedReader extends BetterPipedReader {
        protected synchronized void receivedFirst() {
            super.receivedFirst();
            try {
                //write a redirect to the underlying response
                if (logger.isInfoEnabled()) logger.info("reader received data...redirecting client to get results");
                if (uresp!=null) LongRunningEventGateway.redirectBrowserToGetLongRunningResults(lrid, uresp);
            } catch (IOException e) {
                logger.warn("Unexpected IOException trying to redirect browser for long running response: "+e, e);            
            }
        }
    }
    
    class CustomizedBetterPipedInputStream extends BetterPipedInputStream {
        protected synchronized void receivedFirst() {
            super.receivedFirst();
            try {
                //write a redirect to the underlying response
                if (logger.isInfoEnabled()) logger.info("reader received data...redirecting client to get results");
                if (uresp!=null) LongRunningEventGateway.redirectBrowserToGetLongRunningResults(lrid, uresp);
            } catch (IOException e) {
                logger.warn("Unexpected IOException trying to redirect browser for long running response: "+e, e);            
            }
        }
    }
    
    Reader getReader() throws IOException {
        return pr;
    }

    /**
     * Returns a boolean indicating if the response has been committed.
     */
    public boolean isCommitted() {
        return (pr!=null);
    }

    /**
     * Clears any data that exists in the buffer as well as the status 
     * code and headers.
     */
    public void reset() {
        _resetVars();
    }

    /**
     * Clears the content of the underlying buffer in the response without 
     * clearing headers or status code.
     */
    public void resetBuffer() {
        pw = null;
        pr = null;
        gotStream = false;
        gotWriter = false;
    }

    /**
     * Sets the preferred buffer size for the body of the response. 
     * Doesn't do anything in this implementation
     */
    public void setBufferSize(int size) {
        bufferSize = size;
    }

    /**
     * Sets the length of the content body in the response In HTTP servlets, 
     * this method sets the HTTP Content-Length header.
     */
    public void setContentLength(int len) {
        contentLength = len;
    }

    /**
     * Sets the content type of the response being sent to the client.
     */
    public void setContentType(String type) {
        contentType = type;
    }

    /**
     * Sets the locale of the response, setting the headers 
     * (including the Content-Type's charset) as appropriate.
     */
    public void setLocale(Locale loc) {
        locale = loc;
    }

    //--------------- HttpServletResponseWrapper -----------------
    /**
     * Adds the specified cookie to the response.
     */
    public void addCookie(Cookie cookie) {
        if (cookies==null) cookies = new ArrayList();
        cookies.add(cookie);
    }
    
    /**
     * Adds a response header with the given name and date-value.
     */
    public void addDateHeader(String name, long date) {
        addHeader(name, sdf.format(new Date(date)));
    }


    /**
     * Adds a response header with the given name and value.
     */
    public void addHeader(String name, String value) {
        if (headers==null) headers = new TreeMap();
        List values = (List) headers.get(name);
        if (values==null) {
            values = new ArrayList();
            headers.put(name, values);
        } else {
            values.add(value);
        }
    }

    /**
     * Adds a response header with the given name and integer value.
     */
    public void addIntHeader(String name, int value) {
        addHeader(name, ""+value);
    }

    /**
     * Returns a boolean indicating whether the named response header 
     * has already been set.
     */
    public boolean containsHeader(String name) {
        return (headers!=null && headers.containsKey(name));
    }

    /**
     * Deprecated. As of version 2.1, use encodeRedirectURL(String url) 
     * instead
     */
    public String encodeRedirectUrl(String url) {
        return encodeRedirectURL(url);
    }

    /**
     * Encodes the specified URL for use in the sendRedirect method or, 
     * if encoding is not needed, returns the URL unchanged.
     */
    public String encodeRedirectURL(String url) {
        return uresp.encodeRedirectURL(url);
    }

    /**
     * Deprecated. As of version 2.1, use encodeURL(String url) instead
     */
    public String encodeUrl(String url) {
        return encodeURL(url);
    }

    /**
     * Encodes the specified URL by including the session ID in it, or, 
     * if encoding is not needed, returns the URL unchanged.
     */
    public String encodeURL(String url) {
        return uresp.encodeURL(url);
    }

    /**
     * Sends an error response to the client using the specified status 
     * code and clearing the buffer.
     */
    public void sendError(int sc) {
        sendError(sc, getStatusMessage(sc));
    }

    /**
     * Sends an error response to the client using the specified status 
     * clearing the buffer.
     */
    public void sendError(int sc, String msg) {
        err = sc;
        errmsg = msg;
    }

    /**
     * Sends a temporary redirect response to the client using the specified 
     * redirect location URL.
     */
    public void sendRedirect(String location) {
        redirectLoc = location;
    }

    /**
     * Sets a response header with the given name and date-value.
     */
    public void setDateHeader(String name, long date) {
        setHeader(name, sdf.format(new Date(date)));
    }

    /**
     * Sets a response header with the given name and value.
     */
    public void setHeader(String name, String value) {
        if (headers==null) headers = new TreeMap();
        List values = new ArrayList(1);
        headers.put(name, values);
        values.add(value);
    }

    /**
     * Sets a response header with the given name and integer value.
     */
    public void setIntHeader(String name, int value) {
        setHeader(name, ""+value);
    }

    /**
     * Sets the status code for this response.
     */
    public void setStatus(int isc) {
        sc = isc;
    }

    /**
     * Sets the status code for this response.
     */
    public void setStatus(int isc, String ism) {
        sc = isc;
        scmsg = ism;    
    }

    /**
     * Returns a default status message for the specified HTTP status code.
     *
     * @param status The status code for which a message is desired
     */
    public static String getStatusMessage(int isc) {
        switch (isc) {
        case SC_OK:
            return ("OK");
        case SC_ACCEPTED:
            return ("Accepted");
        case SC_BAD_GATEWAY:
            return ("Bad Gateway");
        case SC_BAD_REQUEST:
            return ("Bad Request");
        case SC_CONFLICT:
            return ("Conflict");
        case SC_CONTINUE:
            return ("Continue");
        case SC_CREATED:
            return ("Created");
        case SC_EXPECTATION_FAILED:
            return ("Expectation Failed");
        case SC_FORBIDDEN:
            return ("Forbidden");
        case SC_GATEWAY_TIMEOUT:
            return ("Gateway Timeout");
        case SC_GONE:
            return ("Gone");
        case SC_HTTP_VERSION_NOT_SUPPORTED:
            return ("HTTP Version Not Supported");
        case SC_INTERNAL_SERVER_ERROR:
            return ("Internal Server Error");
        case SC_LENGTH_REQUIRED:
            return ("Length Required");
        case SC_METHOD_NOT_ALLOWED:
            return ("Method Not Allowed");
        case SC_MOVED_PERMANENTLY:
            return ("Moved Permanently");
        case SC_MOVED_TEMPORARILY:
            return ("Moved Temporarily");
        case SC_MULTIPLE_CHOICES:
            return ("Multiple Choices");
        case SC_NO_CONTENT:
            return ("No Content");
        case SC_NON_AUTHORITATIVE_INFORMATION:
            return ("Non-Authoritative Information");
        case SC_NOT_ACCEPTABLE:
            return ("Not Acceptable");
        case SC_NOT_FOUND:
            return ("Not Found");
        case SC_NOT_IMPLEMENTED:
            return ("Not Implemented");
        case SC_NOT_MODIFIED:
            return ("Not Modified");
        case SC_PARTIAL_CONTENT:
            return ("Partial Content");
        case SC_PAYMENT_REQUIRED:
            return ("Payment Required");
        case SC_PRECONDITION_FAILED:
            return ("Precondition Failed");
        case SC_PROXY_AUTHENTICATION_REQUIRED:
            return ("Proxy Authentication Required");
        case SC_REQUEST_ENTITY_TOO_LARGE:
            return ("Request Entity Too Large");
        case SC_REQUEST_TIMEOUT:
            return ("Request Timeout");
        case SC_REQUEST_URI_TOO_LONG:
            return ("Request URI Too Long");
        case SC_REQUESTED_RANGE_NOT_SATISFIABLE:
            return ("Requested Range Not Satisfiable");
        case SC_RESET_CONTENT:
            return ("Reset Content");
        case SC_SEE_OTHER:
            return ("See Other");
        case SC_SERVICE_UNAVAILABLE:
            return ("Service Unavailable");
        case SC_SWITCHING_PROTOCOLS:
            return ("Switching Protocols");
        case SC_UNAUTHORIZED:
            return ("Unauthorized");
        case SC_UNSUPPORTED_MEDIA_TYPE:
            return ("Unsupported Media Type");
        case SC_USE_PROXY:
            return ("Use Proxy");
        case 207:       // WebDAV
            return ("Multi-Status");
        case 422:       // WebDAV
            return ("Unprocessable Entity");
        case 423:       // WebDAV
            return ("Locked");
        case 507:       // WebDAV
            return ("Insufficient Storage");
        default:
            return ("HTTP Response Status " + isc);
        }

    }
          


    /**
     * simply for testing purposes
     */
    public static void main(String args[]) {
        try {
            System.out.println("\nWriting sample data...");
            DeferredResponseWrapper resp = new DeferredResponseWrapper();
            PrintWriter out = resp.getWriter();
/*
            out.println("Test1");
            out.println("Test2");
            out.println("Test3");
*/
            for (int i=0; i<3000; i++) {
//System.out.println("i:"+i);
                out.print("a");
            }
            out.close();

            System.out.println("\nReading sample data...");
            Reader in = resp.getReader();
            int offset = 0;
            int len = 1024;
            char[] data = new char[len];
            while (true) {
                int cnt = in.read(data, 0, len);
                if (cnt==-1) break;
System.out.println("...read: "+cnt+" chars");                
//                System.out.println(new String(data, 0, cnt));
                offset+=cnt;
            }
System.out.println("total: "+offset+" chars");                
            in.close();
            System.out.println("\nFinished!");
    
        } catch (IOException e) {
            System.out.println("Err: "+e);
            e.printStackTrace();
        }
    }
          
}

