/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: CopyAndReplace.java 125 2006-01-02 20:59:13Z christianc $
 */
package org.barracudamvc.taskdefs;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.Copy;
import org.apache.tools.ant.taskdefs.Replace;
import org.apache.tools.ant.types.EnumeratedAttribute;

/**
 * <p>A consolidated copy and replace task that extends the basic Ant Copy
 * taskdef.
 *
 * <p>In addition, once files have been copied, a replace function is also
 * run on them. This replace will only affect the files that actually
 * get copied; it will not affect any other files in the directory.
 *
 * <p>The replace occurs using a simply properties file, rather than
 * specifying replace parameters via the xml. The format of this file
 * looks something like this:
 *
 * <ul>
 *   <li>token="some target text" value="some replacement text"</li>
 *   <li>token='some target text called "blah"' value='some replacement text named "blech"'</li>
 *   <li>token=~some target text with " and '~ value=^some replacement text with ' and "^</li>
 * </ul>
 *
 * <p>As you can see, the format is flexible. Tokens are identified with "token=" and
 * values are identified by "value=". The actual token/value delimiters are
 * taken to be the first character following the = sign. This could be a double
 * quote, single quote, or any other character (ie. if the text you wish to
 * replace contains both double and single quotes, you might want to use a ~
 * delimiter or something like that).
 *
 * <p>Note also that this taskdef supports SSI replacement, which can be quite useful.
 * By default, the copied files are NOT processed for SSIs; if you want to use this,
 * turn is on using the 'ssi=true' attribute. SSI replacement occurs BEFORE the token
 * replacement (ensuring that the text contained in the SSI gets processed for tokens
 * as well).
 *
 * <p>If SSI replacement is turned on, you can also exclude blocks of code using #exclude_start
 * and #exclude_end tags (as many as you want in the template). This allows you to drop code
 * from the mockups when you copy across. For example:
 *
 * <br>    <!--#exclude_start--><p>blah blah blah<!--#exclude_end-->
 * <br>will result in this after copying:
 * <br>    <!-- -->
 *
 * <p>As a final note, doing SSI replacement on a hierarchy of directories can easily cause
 * broken link references (ie. if your SSI is in the root directory, and it refers to images/foo.gif,
 * then when you suck that SSI into a file several directories deep (eg. /foo/bar/blah.html), then
 * that ssi reference really needs to get changed to ../../images/foo.gif. You can do this by specifying
 * a relative dir constant in your options.mappings file:
 *
 * <br>    token=^"images/^ value=^"@REL_PATH@images/^
 *
 * @author Christian Cryder [christianc@granitepeaks.com]
 */
public class CopyAndReplace extends Copy {

    public final static String REL_PATH_TOKEN = "@REL_PATH@";   //csc_020505_2
    
    protected File mappingsFile = null; //the mappings file
    protected boolean ssi = false;      //process for ssi's         //csc_111601.1 - added

    /**
     * Sets the mappings file.
     */
    public void setMappings(File mappingsFile) {
        this.mappingsFile = mappingsFile;
    }

    //csc_111601.1 - added
    /**
     * Process files copied for ssi
     *
     * @param issi set "true" to process copied files for ssi
     */
    public void setSsi(BooleanAttribute issi) {
        ssi = (issi.getValue().equals("yes") || issi.getValue().equals("true"));
    }

    /**
     * processSSI is the result of a code refactoring due to changes in Ant 1.5 to Ant 1.6
     * Ant 1.6 supports multiple target file, therefore processSSI might be called n-times
     * @param fromFile
     * @param toFile
     */
    protected void processSSI(String fromFile, String toFile) {
        File targetFile = new File(toFile);
        File fromDir = new File(fromFile).getParentFile();

        //get the file (this will automatically resolve ssi's)
        byte[] dstbytes = getFile(targetFile, fromDir);
        if (dstbytes==null) return;

        //now write the file back out
        try {
            FileOutputStream fos = new FileOutputStream(targetFile);
            fos.write(dstbytes);
            fos.close();
        } catch (IOException e) {
            System.out.println("Error writing file " + targetFile + ":" + e);
        }
    }

    /**
     * perform the token-value based replacement. This method was introduced due to changes
     * from Ant 1.5 to Ant 1.6. It might be called multiple times
     * @param fromFile
     * @param toFile
     * @param tokenList
     * @param valueList
     */
    protected void processReplace(String fromFile, String toFile, List tokenList, List valueList) {
        File targetFile = new File(toFile);
//System.out.println("replace: destDir:"+this.destDir+" fromFile:"+fromFile+" toFile:"+toFile);

        //do the replace on each file (this is important: we only
        //want to do the replace on files we actually copied in)
        Replace replace = new Replace();
        replace.setProject(this.getProject());
        replace.setFile(targetFile);
        for (int i=0, max=tokenList.size(); i<max; i++) {
            Replace.Replacefilter rf = replace.createReplacefilter();
            rf.setToken((String) tokenList.get(i));
            rf.setValue((String) valueList.get(i));
        }

//csc_020505_2_start
        //figure out the relative path token (so that we can reference this in the mappings file)
        String relPath = "";
        File parentDir = (targetFile!=null ? targetFile.getParentFile() : null);
        while (parentDir!=null && parentDir.compareTo(this.destDir)!=0) {
            relPath = relPath+"../";
            parentDir = parentDir.getParentFile();
        }
//System.out.println("relPath:"+relPath);        
        Replace.Replacefilter rf = replace.createReplacefilter();
        rf.setToken(REL_PATH_TOKEN);
        rf.setValue(relPath);
//csc_020505_2_end

        //do the deed and make the subsitutions
        replace.execute();
    }

    /**
     * <p>Actually does the file (and possibly empty directory) copies.
     * This is a good method for subclasses to override.
     *
     * <p>Note that all the copy functionality occurs by simply deferring
     * to the superclass implementation. The replace functionality follows:
     *
     * <ul>
     *    <li><p>first we make sure there is a mappings file</li>
     *    <li><p>next we parse it to determine all token/value mapings</li>
     *    <li><p>finally we iterate through the list of files that actually
     *          got copied and we create Replace task for each of them. This
     *          task contains all the various token/value mappings, and
     *          gets executed for each file, effectively making all the
     *          necessary text substitutions</li>
     * <ul>
     */
    protected void doFileOperations() {
        //start by allowing the basic copy to occur
        super.doFileOperations();

        //csc_082802.1_start - move the following block of code up from down below.
        //This was done in response to a patch submitted by Stefan Armbruster [sarmbruster@web.de]
        //which allows CopyAndReplace to work if you only want to do SSI processing,
        //and don't need to do any mapping. This change means that you no longer have
        //to supply an empty mappings file.

        //support ssi
        if (ssi) {
            if (fileCopyMap.size()>0) log("Processing copied files for server side includes");

            Enumeration enumeration = fileCopyMap.keys();
            while (enumeration.hasMoreElements()) {
                //figure out the target file
                String fromFile = (String) enumeration.nextElement();

                //sam_2003-08-13: Ant 1.5 returns a String, Ant 1.6 returns a String[] (multiple targets)
                Object target = fileCopyMap.get(fromFile);
                boolean isAnt15 = target instanceof String;
                if (isAnt15) {
                    processSSI(fromFile, (String)target);
                } else {
                    String[] targetArray = (String[]) target;
                    for (int j = 0; j < targetArray.length; j++) {
                        processSSI(fromFile, targetArray[j]);
                    }
                }
                //sam_2003-08-13: end

            }
        }
        //csc_082802.1_end

        //if the mappings file is null or doesn't exist just return
        if (mappingsFile==null) return;
        if (!mappingsFile.exists()) {
            log("Unable to find mappings file: "+mappingsFile+" ... files copied but no text replace occurred", Project.MSG_ERR);
            return;
        }

        //start by reading the mappings file
        List tokenList = new ArrayList();
        List valueList = new ArrayList();
        try {
            BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(mappingsFile)));
            while (true) {
                String s = br.readLine();
                if (s==null) break;
                String token = null;
                String value = null;

                //get the token
                int spos = s.indexOf("token=");
                if (spos<0 || spos+6>=s.length()) continue;
                spos+=6;
                String delim = s.substring(spos,spos+1);
                spos+=1;
                int epos = s.indexOf(delim, spos+1);
                if (epos<0 || epos+1>=s.length()) continue;
                token = s.substring(spos, epos);
                if (token==null) continue;

                //get the value
                spos = s.indexOf("value=", epos+1);
                if (spos<0 || spos+6>=s.length()) continue;
                spos+=6;
                delim = s.substring(spos,spos+1);
                spos+=1;
                epos = s.indexOf(delim, spos+1);
                if (epos<0) continue;
                value = s.substring(spos, epos);
                if (value==null) continue;

                tokenList.add(token);
                valueList.add(value);
            }
        } catch (IOException e) {
            System.out.println ("Error reading file "+mappingsFile+":"+e);
            return;
        }

        //now iterate through the file copy list
        if (fileCopyMap.size()>0) {
            log("Replacing text in copied files");

            Enumeration enumeration = fileCopyMap.keys();
            while (enumeration.hasMoreElements()) {
                //figure out the target file
                String fromFile = (String) enumeration.nextElement();

                // sam,2003-08-13: Ant 1.5 returns a String, Ant 1.6 returns a String[] (multiple targets)
                Object target = fileCopyMap.get(fromFile);
                boolean isAnt15 = target instanceof String;
                if (isAnt15) {
                    processReplace(fromFile, (String)target, tokenList, valueList );
                } else {
                    String[] targetArray = (String[]) target;
                    for (int j = 0; j < targetArray.length; j++) {
                        processReplace(fromFile, targetArray[j], tokenList, valueList );
                    }
                }
                // sam_2003-08-13: end
            }
        }
    }

    public static class BooleanAttribute extends EnumeratedAttribute {
        public String[] getValues() {
            return new String[] {"yes", "no", "true", "false"};
        }
    }


    //csc_111601.1_start
    private static String START_SSI = "<!--#include file=\"";
    private static String END_SSI = "\"-->";
    private static String START_EXCLUDE = "#exclude_start";     //csc_020505_1
    private static String END_EXCLUDE = "#exclude_end";         //csc_020505_1

    public byte[] getFile(File targetFile, File fromDir) {
        //adjust the target file
        if (!targetFile.exists() && fromDir!=null) {
            String targetFileName = targetFile.getName();
            targetFile = new File(fromDir, targetFileName);
        }
//System.out.println("...targetFile:"+targetFile+" fromDir:"+fromDir);        

        //get the resource
        byte[] dstbytes = new byte[10240];
        try {
            //read in the file and store it in a byte array
            FileInputStream fis = new FileInputStream(targetFile);
            BufferedInputStream in = new BufferedInputStream(fis);
            int offset = 0;
            int len = 1024;
            byte[] inbytes = new byte[len];
            while (true) {
                int cnt = in.read(inbytes, 0, len);
                if (cnt==-1) break;
                if (offset+cnt>=dstbytes.length) {
                    byte[] newbytes = new byte[(int) (dstbytes.length+(len*5))];
                    System.arraycopy(dstbytes, 0, newbytes, 0, dstbytes.length);
                    dstbytes = newbytes;
                }
                System.arraycopy(inbytes, 0, dstbytes, offset, cnt);
                offset+=cnt;
            }
            in.close();

            //when we're done trim off the excess
            byte[] newbytes = new byte[offset];
            System.arraycopy(dstbytes, 0, newbytes, 0, offset);
            dstbytes = newbytes;

            //now create a new string from the result, and process it for ssi's
            //(this is not exactly the most efficient way of doing things, but it'll
            //work for now...if you don't like it, optimize it and submit it back
            //to the group ;-)
            String result = new String(dstbytes);
            int spos = 0;
            int epos = 0;
            while (true) {
                spos = result.indexOf(START_SSI, epos);
                if (spos<0) break;
                epos = result.indexOf(END_SSI, spos);
                if (epos<spos) break;
                String ssiName = result.substring(spos+START_SSI.length(), epos);
                File ssiFile = null;
                if (ssiFile==null || !ssiFile.exists()) ssiFile = new File(targetFile.getParent(), ssiName);
                if (ssiFile==null || !ssiFile.exists()) ssiFile = new File(fromDir, ssiName);
                if (ssiFile==null || !ssiFile.exists()) ssiFile = new File(ssiName);
//csc_020505_1_start
                //added so that we can handle subdirs. Basically, if the file is still not found,
                //chain back up until we hit the dir where the mappings file lives (for performance 
                //reasons, don't go higher if we haven't found it by then)
                if (ssiFile==null || !ssiFile.exists()) {
                    File startDir = fromDir;
                    File topDir = (mappingsFile!=null ? mappingsFile.getParentFile() : null);
                    while (startDir.getParent()!=null) {
                        ssiFile = new File(startDir.getParent(), ssiName);
                        if (ssiFile.exists()) break;
                        startDir = startDir.getParentFile();
                        if (topDir!=null && topDir.compareTo(startDir)==0) break;
                    }
                }
//csc_020505_1_end
                if (ssiFile==null || !ssiFile.exists()) {
                    System.out.println ("Error locating SSI:"+ssiName);
                    continue;
                }
                byte[] ssiBytes = getFile(ssiFile, null);
//csc_010206_1_start
//need to make sure that we adjust epos correctly or some include statements get skipped
//                if (ssiBytes!=null) result = result.substring(0,spos)+new String(ssiBytes)+result.substring(epos+END_SSI.length());
                if (ssiBytes!=null) {
                    String istr = new String(ssiBytes);
                    result = result.substring(0,spos)+istr+result.substring(epos+END_SSI.length());
                    epos = spos+istr.length();
                }
//csc_010206_1_end
            }
            
//csc_020505_1_start            
            //now once we're all done, we need to get rid of any exclude blocks
            spos = result.indexOf(START_EXCLUDE);
            if (spos>-1) {
                StringBuffer sb = new StringBuffer(result);
                while (spos>-1) {
                    epos = sb.indexOf(END_EXCLUDE, spos);
                    if (epos<0) break;
                    sb.replace(spos, epos+END_EXCLUDE.length(), " ");
                    spos = sb.indexOf(START_EXCLUDE, spos);
                }
                result = sb.toString();            
            }
//csc_020505_1_end
            
            dstbytes = result.getBytes();

        } catch (IOException e) {
            System.out.println ("Error reading file "+targetFile+":"+e);
            return null;
        }
        return dstbytes;
    }
    //csc_111601.1_end
}
