/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: TestExample.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc;

import org.apache.log4j.*;
import org.barracudamvc.testbed.*;


/**
 * Just a little sample test to illustrate how it all works, Runtime parameters
 * are defined in TestUtil.java. We are using basic Log4J logging.
 * 
 * You can run individual test classes directly, or you can run the entire test 
 * suite using Ant. You control what gets logged by modifying the values in
 * /WEB-INF/log4j.xml
 *
 * If you wish to create custom test classes, you should use this class as a 
 * model. 
 */
public class TestExample extends DefaultTestCase {
    //common vars (customize for every test class)
    private static final String testClass = TestExample.class.getName();
    private static final Logger logger = Logger.getLogger("test."+testClass);

    //variables
    private static boolean demonstrateFailure = false;        //change this to true to see how a failure is handled
    private static boolean demonstrateException = false;    //change this to true to see how an exception is handled
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestExample(String name) {
        super(name);
    }
    
    /**
     * Every test class should have a main method so it can be run 
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    
    //-------------------- TestCase ------------------------------
    /**
     * Set up the tests
     */
    protected void setUp() throws Exception {
        if (logger.isDebugEnabled()) logger.debug("Setting up test case");
        super.setUp();
        
        //this is where we can do any setup prior to 
        //actually running the tests if we so desire
        //(--n/a--)
    }

    /**
     * Tear down after the tests
     */
    protected void tearDown() throws Exception {
        if (logger.isDebugEnabled()) logger.debug("Tearing down test case");
        super.tearDown();
        
        //this is where we can do any cleanup after 
        //actually running the tests if we so desire
        //(--n/a--)
    }
    
    
    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!
    
    /**
     * Simple test to demonstrate how we can test various scenarios
     * and then log any errors which occur.
     */
    public void testSample() {
        if (logger.isInfoEnabled()) logger.info("running testSample()");

        //sample assertions (if an assertion fails then the error 
        //message will be thrown)
        try {
            //assert true!=false
            if (logger.isInfoEnabled()) logger.info("testing true!=false");
            assertTrue("true not equal false", true!=false);

            //assert 1==0
            if (demonstrateFailure) {
                if (logger.isInfoEnabled()) logger.info("testing intentional failure");
                assertTrue("intentionally failed assertion", 1==0);
            }
            
            //sample exception
            if (demonstrateException) {
                if (logger.isInfoEnabled()) logger.info("generating sample exception");
                throw new Exception("Sample Exception");
            }

            //finished successfully!
            if (logger.isInfoEnabled()) logger.info("tests complete!");
        } catch (Exception e) {
            //here's where we should catch unexpected exceptions and 
            //manually cause the test to fail
            logger.error("Unexpected Test Exception", e);
            fail("Unexpected Test Exception:"+e);
        }
    }
}
