/*
 * Copyright (C) 2004  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: TestDefaultFormMap.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.core.forms;

import java.math.*;
import java.sql.*;
import java.text.*;
import java.util.*;
import java.util.Date;

import org.apache.log4j.*;
import org.barracudamvc.testbed.*;
import org.barracudamvc.plankton.data.*;

/**
 * Test the DefaultFormMap class to make sure it maps correctly
 */
public class TestDefaultFormMap extends DefaultTestCase {
    //common vars (customize for every test class)
    private static final String testClass = TestDefaultFormMap.class.getName();
    private static final Logger logger = Logger.getLogger("test."+testClass);

    //variables
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestDefaultFormMap(String name) {
        super(name);
    }
    
    /**
     * Every test class should have a main method so it can be run 
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    
    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!
    
    /**
     * Simple test to demonstrate how we can test various scenarios
     * and then log any errors which occur.
     */
    public void testMap() throws java.text.ParseException {
        if (logger.isInfoEnabled()) logger.info("testing form mapping");

        //common vars        
        FormMap fm = null;
        FormMapper fmMapper = null;
        Map mappedElements = null;
        Map keymap = null;
        DateFormat df = DateFormat.getDateInstance();
        DateFormat df2 = DateFormat.getTimeInstance();
        Date sampleDate = df.parse(df.format(new Date()));
        Timestamp sampleTimestamp = new Timestamp(new Date().getTime());
        Time sampleTime = new Time(df2.parse(df2.format(new Date())).getTime());
        Object map_yes[][] = null;
        Object map_no[][] = null;
//System.out.println("Here 0:"+((SimpleDateFormat) df2).toPattern());
//System.out.println("sampleDate:"+sampleDate);
//System.out.println("sampleTime:"+sampleTime.getTime());
        
        
        //test basic form mapping
        map_yes = new Object[][] { 
            {"foo1", "blah", FormType.STRING, "blah"},
            {"foo2.1", "on", FormType.BOOLEAN, Boolean.TRUE},
            {"foo2.2", "yes", FormType.BOOLEAN, Boolean.TRUE},
            {"foo2.3", "true", FormType.BOOLEAN, Boolean.TRUE},
            {"foo2.4", "Y", FormType.BOOLEAN, Boolean.TRUE},
            {"foo2.5", "off", FormType.BOOLEAN, Boolean.FALSE},
            {"foo2.6", "no", FormType.BOOLEAN, Boolean.FALSE},
            {"foo2.7", "false", FormType.BOOLEAN, Boolean.FALSE},
            {"foo2.8", "N", FormType.BOOLEAN, Boolean.FALSE},
            {"foo3", "1", FormType.INTEGER, new Integer(1)},
            {"foo4", "2", FormType.LONG, new Long(2)},
            {"foo5", "3", FormType.SHORT, new Short((short)3)},
            {"foo6", "4", FormType.DOUBLE, new Double(4)},
            {"foo7", "5.0", FormType.FLOAT, new Float(5.0)},
            {"foo8", "6.1234", FormType.BIG_DECIMAL, new BigDecimal("6.1234")},
            {"foo9", df.format(sampleDate), FormType.DATE, sampleDate},
            {"foo10", ""+sampleTimestamp, FormType.TIMESTAMP, sampleTimestamp},
            {"foo11", ""+sampleTime, FormType.TIME, sampleTime},
        };
        map_no = new Object[][] { 
            {"foo97", "blah1"},
            {"foo98", "blah2"},
            {"foo99", "blah3"},
        };
        fm = new DefaultFormMap();
        keymap = new TreeMap();
        for (int i=0; i<map_yes.length; i++) {
            keymap.put(map_yes[i][0], map_yes[i][1]);
            fm.defineElement(new DefaultFormElement((String) map_yes[i][0], (FormType) map_yes[i][2]));
        }
        for (int i=0; i<map_no.length; i++) {
            keymap.put(map_no[i][0], map_no[i][1]);
        }
//CollectionsUtil.printStackTrace(keymap);        
        fm.map(new MapStateMap(keymap));
        mappedElements = fm.getFormMapper().getElements();
//CollectionsUtil.printStackTrace(mappedElements);
        for (int i=0; i<map_yes.length; i++) {
            String key = (String) map_yes[i][0];
            String textVal = (String) map_yes[i][1];
            FormType formType = (FormType) map_yes[i][2];
            Object targetVal = map_yes[i][3];
            assertTrue("key '"+key+"' was not mapped (and it should have been)", mappedElements.containsKey(key));
            FormElement el = fm.getElement(key);
            assertTrue("key '"+key+"' did not return the expected form element", el!=null);
            assertTrue("key '"+key+"' mapped to '"+el.getVal()+"' instead of '"+targetVal+"' as expected", targetVal.equals(el.getVal()));
            assertTrue("key '"+key+"' origVal mapped to '"+el.getOrigVal()+"' instead of '"+textVal+"' as expected", textVal.equals(el.getOrigVal()));
            assertTrue("key '"+key+"' type mapped to '"+el.getType()+"' instead of '"+formType+"' as expected", formType.equals(el.getType()));
        }
        for (int i=0; i<map_no.length; i++) {
            String key = (String) map_no[i][0];
            assertTrue("key '"+key+"' was mapped (and it should NOT have been)", !mappedElements.containsKey(key));
        }
        
        
        
        //test prefix form mapping
        map_yes = new Object[][] { 
            {"foo1", "blah", FormType.STRING, "blah"},
            {"foo2", "Y", FormType.BOOLEAN, Boolean.TRUE},
            {"foo3", "1", FormType.INTEGER, new Integer(1)},
        };
        map_no = new Object[][] { 
            {"foo4", "2", FormType.INTEGER, new Integer(2)},
            {"foo5", "3", FormType.INTEGER, new Integer(3)},
        };
        String prefix = "pr_";        
        fmMapper = new PrefixFormMapper(prefix);
        fm = new DefaultFormMap();
        fm.setFormMapper(fmMapper);
        keymap = new TreeMap();
        for (int i=0; i<map_yes.length; i++) {
            keymap.put(prefix+map_yes[i][0], map_yes[i][1]);
            fm.defineElement(new DefaultFormElement((String) map_yes[i][0], (FormType) map_yes[i][2]));
        }
        for (int i=0; i<map_no.length; i++) {
            keymap.put(map_no[i][0], map_no[i][1]);
            fm.defineElement(new DefaultFormElement((String) map_no[i][0], (FormType) map_no[i][2]));
        }
//CollectionsUtil.printStackTrace(keymap);        
        fm.map(new MapStateMap(keymap));
        mappedElements = fm.getFormMapper().getElements();
//CollectionsUtil.printStackTrace(mappedElements);
        for (int i=0; i<map_yes.length; i++) {
            String key = (String) map_yes[i][0];
            String textVal = (String) map_yes[i][1];
            FormType formType = (FormType) map_yes[i][2];
            Object targetVal = map_yes[i][3];
            assertTrue("key '"+key+"' was not mapped (and it should have been)", mappedElements.containsKey(key));
            FormElement el = fm.getElement(key);
            assertTrue("key '"+key+"' did not return the expected form element", el!=null);
            assertTrue("key '"+key+"' mapped to '"+el.getVal()+"' instead of '"+targetVal+"' as expected", targetVal.equals(el.getVal()));
            assertTrue("key '"+key+"' origVal mapped to '"+el.getOrigVal()+"' instead of '"+textVal+"' as expected", textVal.equals(el.getOrigVal()));
            assertTrue("key '"+key+"' type mapped to '"+el.getType()+"' instead of '"+formType+"' as expected", formType.equals(el.getType()));
        }
        for (int i=0; i<map_no.length; i++) {
            String key = (String) map_no[i][0];
            assertTrue("key '"+key+"' was mapped (and it should NOT have been)", !mappedElements.containsKey(key));
        }


        
        
        //test uid form mapping
        String[] uids = new String[] { 
            "1234", "1235", "1236",
        };

        map_yes = new Object[][] { 
            {"foo1", "blah", FormType.STRING, "blah"},
            {"foo2", "Y", FormType.BOOLEAN, Boolean.TRUE},
            {"foo3", "1", FormType.INTEGER, new Integer(1)},
        };
        map_no = new Object[][] { 
            {"foo97", "blah1"},
            {"foo98", "blah2"},
            {"foo99", "blah3"},
        };
        fmMapper = new UIDFormMapper();
        fm = new DefaultFormMap();
        fm.setFormMapper(fmMapper);
        keymap = new TreeMap();
        for (int i=0; i<map_yes.length; i++) {
            for (int j=0; j<uids.length; j++) {
                keymap.put(map_yes[i][0]+UIDFormMapper.UID_TOKEN+uids[j], map_yes[i][1]);
            }
            fm.defineElement(new DefaultFormElement((String) map_yes[i][0], (FormType) map_yes[i][2]));
        }
        for (int i=0; i<map_no.length; i++) {
            keymap.put(map_no[i][0], map_no[i][1]);
        }
//CollectionsUtil.printStackTrace(keymap);        
        fm.map(new MapStateMap(keymap));
        mappedElements = fm.getFormMapper().getElements();
//CollectionsUtil.printStackTrace(mappedElements);
        Set mappedUIDs = ((UIDFormMapper) fmMapper).getMappedUIDs();
//CollectionsUtil.printStackTrace(new ArrayList(mappedUIDs));
        for (int i=0; i<map_yes.length; i++) {
            String okey = (String) map_yes[i][0];
            String textVal = (String) map_yes[i][1];
            FormType formType = (FormType) map_yes[i][2];
            Object targetVal = map_yes[i][3];
            for (int j=0; j<uids.length; j++) {
                String key = okey+UIDFormMapper.UID_TOKEN+uids[j];
                assertTrue("key '"+key+"' was not mapped (and it should have been)", mappedElements.containsKey(key));
                FormElement el = fm.getElement(key);
                assertTrue("key '"+key+"' did not return the expected form element", el!=null);
                assertTrue("key '"+key+"' mapped to '"+el.getVal()+"' instead of '"+targetVal+"' as expected", targetVal.equals(el.getVal()));
                assertTrue("key '"+key+"' origVal mapped to '"+el.getOrigVal()+"' instead of '"+textVal+"' as expected", textVal.equals(el.getOrigVal()));
                assertTrue("key '"+key+"' type mapped to '"+el.getType()+"' instead of '"+formType+"' as expected", formType.equals(el.getType()));
                assertTrue("okey '"+okey+"' unable to lookup element by okey and uid as expected", el.equals(((UIDFormMapper) fmMapper).getElementByUID(fm, okey, uids[j])));
            }
        }
        assertTrue("Not all the uids got mapped @1: "+mappedUIDs, Arrays.asList(uids).containsAll(mappedUIDs));
        assertTrue("Not all the uids got mapped @2: "+mappedUIDs, mappedUIDs.containsAll(Arrays.asList(uids)));
        for (int i=0; i<map_no.length; i++) {
            String key = (String) map_no[i][0];
            assertTrue("key '"+key+"' was mapped (and it should NOT have been)", !mappedElements.containsKey(key));
        }

    }
}
